/*
 * $Id: Externalizable.java,v 1.3 1998/10/26 01:46:04 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.io/Externalizable.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/26 01:46:04 $
 * $State: Exp $
 */
package au.net.aba.io;

import java.io.*;

/**
 * A compatibility class which provides an externalisable interface
 */
public interface Externalizable extends Serializable
{
	/**
	 * The object implements the writeExternal method to save its contents
	 * by calling the methods of DataOutput for its primitive values or
	 * calling the writeObject method of ObjectOutput for objects, strings
	 * and arrays. 
	 * 
	 * @exception IOException Includes any I/O exceptions that may occur 
	 */
	public abstract void writeExternal(ObjectOutput out)
		throws IOException;

	/**
	 * The object implements the readExternal method to restore its
	 * contents by calling the methods of DataInput for primitive types and
	 * readObject for objects, strings and arrays. The readExternal method
	 * must read the values in the same sequence and with the same types
	 * as were written by writeExternal. 
	 *
	 * @exception IOException Includes any I/O exceptions that may occur 
	 * @exception ClassNotFoundException If the class for an object being
	 * 	restored cannot be found. 
	 */
	 public abstract void readExternal(ObjectInput in)
		throws IOException, ClassNotFoundException;
}
