/*
 * $Id: ObjectInput.java,v 1.9 1998/10/27 05:10:45 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.io/ObjectInput.java,v $
 * $Revision: 1.9 $
 * $Date: 1998/10/27 05:10:45 $
 * $State: Exp $
 */
package au.net.aba.io;

import java.io.*;
import java.util.*;

/**
 * A compatibility class which reads objects serialised using the
 * externalisable interface.
 */
public class ObjectInput extends DataInputStream implements StreamConstants
{
	Hashtable	objects;

	int		objectCount;

	public ObjectInput(
		InputStream	in)
		throws IOException, StreamCorruptedException
	{
		super(in);

		objectCount = 0;
		objects = new Hashtable();

		/*
		 * read the stream header
		 */
		short	ver;

		if (readShort() != STREAM_MAGIC)
		{
			throw new StreamCorruptedException(
				"Bad magic number in ObjectInput.");
		}

		if ((ver = readShort()) != STREAM_VERSION)
		{
			throw new StreamCorruptedException(
				"Bad version got " + ver + " not " 
					+ STREAM_VERSION);
		}
	}

	/**
	 * Read in an object. As we are expecting an externalised stream
	 * the only things we have to worry about are repeated objects
	 * and class references.
	 *
	 * @exception ClassNotFoundException	If the calss of a serialised
	 * object cannot be found.
	 */
	public Object readObject()
		throws IOException, ClassNotFoundException,
			OptionalDataException
	{
		/*
		 * object header
		 */
		String	className;
		byte	code1, code2;

		code1 = (byte)read();	
/*
		// THIS CODE IS BROKEN!!

		if (code1 == TC_REFERENCE)
		{
			int	ref;
			Object	obj;

			ref = readInt() - BASE_REF_HANDLE;

			obj = objects.get(new Integer(ref));
			if (obj == null)
			{
				throw new IOException(
					"Unable to resolve reference to obj for id " + ref);
			}

			return obj;
		}
*/

		if (code1 == TC_NULL)
		{
			return null;
		}

		if (code1 != TC_OBJECT)
		{
			throw new IOException(
					"Unexpected marker in stream.");
		}

		code2 = (byte)read();		// write(TC_CLASSDESC);

		className = null;

		if (code2 == TC_CLASSDESC)
		{
			className = readUTF();

			objects.put(new Integer(objectCount++), className);

			/*
			 * serialVersionUID - ignore
			 */
			readLong();

			/*
			 * classDescFlags
			 */
			read();		// write(SC_EXTERNALIZABLE);

			/*
			 * fields (always zero)
			 */
			readShort();	// writeShort(0);

			/*
			 * skip end block data marker
			 */
			read(); 	

			/*
			 * no super class descriptor either - output a null
			 */
			read();		// write(TC_NULL);
		}
		else if (code2 == TC_REFERENCE)
		{
			int	ref;

			ref = readInt() - BASE_REF_HANDLE;

			className = (String)objects.get(new Integer(ref));
			if (className == null)
			{
				throw new IOException(
					"Unable to resolve reference to name for id " + ref);
			}
		}
		else
		{
			throw new IOException("Dud input stream.");
		}

		/*
		 * save the classname in the hash table
		 */
		objects.put(new Integer(objectCount++), className);

		/*
		 * load the object
		 */
		Class		classDesc;
		Externalizable	obj;

		obj = null;
		classDesc = Class.forName(className);

		try
		{
			obj = (Externalizable)classDesc.newInstance();
		}
		catch (Exception e)
		{
			throw new IOException(e.toString());
		}

		obj.readExternal(this);

		return obj;
	}
}
