/*
 * $Id: ObjectOutput.java,v 1.6 1998/10/26 01:46:05 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.io/ObjectOutput.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/26 01:46:05 $
 * $State: Exp $
 */
package au.net.aba.io;

import java.io.*;

/**
 * A compatibility class which writes objects serialised using the
 * externalisable interface.
 *
 * Note: this interface assumes that, if the JVM doing the deserialisation
 * is 1.1 or greater, that the classes implementing Externalizable include
 * the declaration:
 *
 *	 static final long serialVersionUID = 1L;
 *
 * in them. Failure to do so will cause the later JVMs to believe the
 * class files they are using are different from the one this class will
 * produce and result in an exception being thrown.
 */
public class ObjectOutput extends DataOutputStream implements StreamConstants
{
	public ObjectOutput(
		OutputStream	out)
		throws IOException
	{
		super(out);

		writeShort(STREAM_MAGIC);
		writeShort(STREAM_VERSION);
	}

	public void writeObject(
		Object	obj)
		throws IOException
	{
		if (obj == null)
		{
			write(TC_NULL);
			return;
		}

		if (!(obj instanceof Externalizable))
		{
			throw new IOException(obj.toString()
				+ " must implement Externalizable.");
		}

		/*
		 * object header
		 */
		write(TC_OBJECT);
		write(TC_CLASSDESC);
		writeUTF(obj.getClass().getName());

		/*
		 * serialVersionUID (always one)
		 */
		writeLong(1L);		

		/*
		 * classDescFlags
		 */
		write(SC_EXTERNALIZABLE);

		/*
		 * fields (always zero)
		 */
		writeShort(0);

		/*
		 * never any block data in this case, just output end marker
		 */
		write(TC_ENDBLOCKDATA);

		/*
		 * no super class descriptor either - output a null
		 */
		write(TC_NULL);

		((Externalizable)obj).writeExternal(this);
	}
}
