/*
 * $Id: RSAPrivateCrtKeySpec.java,v 1.2 1998/10/26 01:46:15 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security.spec/RSAPrivateCrtKeySpec.java,v $
 * $Revision: 1.2 $
 * $Date: 1998/10/26 01:46:15 $
 * $State: Exp $
 */
package au.net.aba.security.spec;

import au.net.aba.math.BigInteger;


/**
 * A class that provides a specification for a RSA Private Key with 
 * components for using Chinese remainder theorem.
 */
public class RSAPrivateCrtKeySpec implements KeySpec
{
	public final static String ident = "$Id: RSAPrivateCrtKeySpec.java,v 1.2 1998/10/26 01:46:15 leachbj Exp $";

	BigInteger	modulus;
	BigInteger	publicExponent;
	BigInteger	privateExponent;
	BigInteger	primeP;
	BigInteger	primeQ;
	BigInteger	primeExponentP;
	BigInteger	primeExponentQ;
	BigInteger	crtCoefficient;

	public RSAPrivateCrtKeySpec(
		BigInteger	modulus,
		BigInteger	publicExponent,
		BigInteger	privateExponent,
		BigInteger	primeP,
		BigInteger	primeQ,
		BigInteger	primeExponentP,
		BigInteger	primeExponentQ,
		BigInteger	crtCoefficient)
	{
		this.modulus = modulus;
		this.publicExponent = publicExponent;
		this.privateExponent = privateExponent;
		this.primeP = primeP;
		this.primeQ = primeQ;
		this.primeExponentP = primeExponentP;
		this.primeExponentQ = primeExponentQ;
		this.crtCoefficient = crtCoefficient;
	}

	/**
	 * Returns the modulus.
	 */
	public BigInteger getModulus()
	{
		return modulus;
	}

	/**
	 * Returns the public exponent.
	 */
	public BigInteger getPublicExponent()
	{
		return publicExponent;
	}

	/**
	 * Returns the private exponent.
	 */
	public BigInteger getPrivateExponent()
	{
		return privateExponent;
	}

	/**
	 * Returns the primeP.
	 */
	public BigInteger getPrimeP()
	{
		return primeP;
	}

	/**
	 * Returns the primeQ.
	 */
	public BigInteger getPrimeQ()
	{
		return primeQ;
	}

	/**
	 * Returns the primeExponentP.
	 */
	public BigInteger getPrimeExponentP()
	{
		return primeExponentP;
	}

	/**
	 * Returns the primeExponentQ.
	 */
	public BigInteger getPrimeExponentQ()
	{
		return primeExponentQ;
	}

	/**
	 * Returns the crtCoefficient.
	 */
	public BigInteger getCrtCoefficient()
	{
		return crtCoefficient;
	}
}
