/*
 * $Id: AlgorithmParametersSpi.java,v 1.5 1998/10/27 05:11:11 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/AlgorithmParametersSpi.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/27 05:11:11 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;
import java.security.*;

import au.net.aba.security.spec.*;

/**
 * This class defines the Service Provider Interface (SPI) for the
 * AlgorithmParameters class, which is used to manage algorithm
 * parameters. 
 * <p>
 * All the abstract methods in this class must be implemented by each
 * cryptographic service provider who wishes to supply parameter
 * management for a particular algorithm. 
 * 
 * @see AlgorithmParameters
 * @see AlgorithmParameterSpec
 * @see DSAParameterSpec
 */
public abstract class AlgorithmParametersSpi
{
	public final static String ident = "$Id: AlgorithmParametersSpi.java,v 1.5 1998/10/27 05:11:11 leachbj Exp $";

	/**
	 * Initialises this parameters object using the parameters specified in
	 * paramSpec.
	 *
	 * @param paramSpec the parameter specification.
	 * @exception InvalidParameterSpecException if the given parameter
	 * 	specification is inappropriate for the initialisation of this
	 * 	parameter object.
	 */
	protected abstract void engineInit(
		AlgorithmParameterSpec paramSpec)
		throws InvalidParameterSpecException;

	/**
	 * Imports the specified parameters and decodes them according to the
	 * primary decoding format for parameters. The primary decoding
	 * format for parameters is ASN.1, if an ASN.1 specification for this
	 * type of parameters exists.
	 *
	 * @param params the encoded parameters.
	 * @exception IOException on decoding errors
	 */
	protected abstract void engineInit(
		byte[] params)
		throws IOException;

	/**
	 * Imports the parameters from params and decodes them according
	 * to the specified decoding format. If format is null, the primary
	 * decoding format for parameters is used. The primary decoding
	 * format is ASN.1, if an ASN.1 specification for these parameters
	 * exists.
	 *
	 * @param params the encoded parameters.
	 * @param format the name of the decoding format.
	 * @exception IOException on decoding errors
	 */
	protected abstract void engineInit(
		byte[] params,
		String format)
		throws IOException;

	/**
	 * Returns a (transparent) specification of this parameters object.
	 * paramSpec identifies the specification class in which the
	 * parameters should be returned. It could, for example, be
	 * DSAParameterSpec.class, to indicate that the parameters
	 * should be returned in an instance of the DSAParameterSpec class.
	 *
	 * @param paramSpec the the specification class in which the
	 * 	parameters should be returned.
	 * @return the parameter specification.
	 * @exception InvalidParameterSpecException if the requested parameter
	 * 	specification is inappropriate for this parameter object.
	 */
	protected abstract AlgorithmParameterSpec engineGetParameterSpec(
		Class paramSpec)
		throws InvalidParameterSpecException;

	/**
	 * Returns the parameters in their primary encoding format. The
	 * primary encoding format for parameters is ASN.1, if an ASN.1
	 * specification for this type of parameters exists.
	 *
	 * @return the parameters encoded using the specified encoding scheme.
	 * @exception IOException on encoding errors.
	 */
	protected abstract byte[] engineGetEncoded()
		throws IOException;

	/**
	 * Returns the parameters encoded in the specified format. If format
	 * is null, the primary encoding format for parameters is used. The
	 * primary encoding format is ASN.1, if an ASN.1 specification for
	 * these parameters exists.
	 *
	 * @param format the name of the encoding format.
	 * @return the parameters encoded using the specified encoding scheme.
	 * @exception IOException on encoding errors.
	 */
	protected abstract byte[] engineGetEncoded(
		String format)
		throws IOException;

	/**
	 * Returns a formatted string describing the parameters.
	 * 
	 * @return a formatted string describing the parameters.
	 */
	protected abstract String engineToString();
}
