/*
 * $Id: KeyPairGenerator.java,v 1.4 1998/10/27 05:10:46 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/KeyPairGenerator.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/27 05:10:46 $
 * $State: Exp $
 */

package au.net.aba.security;

import au.net.aba.security.spec.*;

/**
 * The KeyPairGenerator class is used to generate pairs of public and
 * private keys. Key pair generators are constructed using the
 * getInstance factory methods (static methods that return instances of
 * a given class).
 *
 * A Key pair generator for a particular algorithm creates a
 * public/private key pair that can be used with this algorithm. It also
 * associates algorithm-specific parameters with each of the generated
 * keys.
 * <p>
 * This class is not incomplete.
 */
public abstract class KeyPairGenerator
{
	public final static String ident = "$Id: KeyPairGenerator.java,v 1.4 1998/10/27 05:10:46 leachbj Exp $";

	String	algorithm;

	/**
	 * Creates a KeyPairGenerator object for the specified algorithm. 
	 */
	protected KeyPairGenerator(
		String	algorithm)
	{
		this.algorithm = algorithm;
	}

	/**
	 * Returns the standard name of the algorithm for this key pair
	 * generator. See Appendix A in the Java Cryptography Architecture
	 * API Specification & Reference for information about standard
	 * algorithm names. 
	 */
	public String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Generates a KeyPairGenerator object that implements the specified
	 * digest algorithm. If the default provider package provides an
	 * implementation of the requested digest algorithm, an instance of
	 * KeyPairGenerator containing that implementation is returned. If
	 * the algorithm is not available in the default package, other
	 * packages are searched..
	 *
	 * @param algorithm - the standard string name of the algorithm. See
	 * 	Appendix A in the Java Cryptography Architecture API Specification
	 * 	& Reference for information about standard algorithm names.
	 * @returns the new KeyPairGenerator object.
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available in the environment.
	 */
	public static KeyPairGenerator getInstance(
		String	algorithm)
	throws NoSuchAlgorithmException
	{
		throw new NoSuchAlgorithmException("Not implemented");
	}

	/**
	 * Generates a KeyPairGenerator object implementing the specified
	 * algorithm, as supplied from the specified provider, if such an
	 * algorithm is available from the provider. 
	 *
	 * @param algorithm - the standard string name of the algorithm. See
	 * 	Appendix A in the Java Cryptography Architecture API Specification
	 * 	& Reference for information about standard algorithm names.
	 * @returns the new KeyPairGenerator object.
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available in the environment.
	 * @exception NoSuchProviderException - if the algorithm is not
	 * 	available in the environment.
	 */
	public static KeyPairGenerator getInstance(
		String	algorithm,
		String	provider)
	throws NoSuchAlgorithmException, NoSuchProviderException
	{
		throw new NoSuchAlgorithmException("Not implemented");
	}

	/**
	 * Initialises the key pair generator for a certain strength using a
	 * system-provided source of randomness and the default parameter set. 
	 */
	public void initialize(
		int	strength)
	{
		initialize(strength, null);
	}

	/**
	 * Initialises the key pair generator for a certain strength,
	 * using the default parameter set. 
	 */
	public abstract void initialize(int strength, SecureRandom random);

	/**
	 * Initialises the key pair generator using the specified parameter set
	 * and user-provided source of randomness. 
	 *
	 * @exception InvalidAlgorithmParameterException - if the provided
	 * 	params is invalid.
	 */
	public void initialize(
		AlgorithmParameterSpec	params,
		SecureRandom		rand)
	throws InvalidAlgorithmParameterException
	{
		/* throw new UnsupportedOperation() */
	}

	/**
	 * Initialises the key pair generator using the specified parameter
	 * set and a system-provided source of randomness. 
	 *
	 * @exception InvalidAlgorithmParameterException - if the provided
	 * 	params is invalid.
	 */
	public void initialize(
		AlgorithmParameterSpec	params)
	throws InvalidAlgorithmParameterException
	{
		initialize(params, null);
	}

	/**
	 * Generates a key pair. Unless an initialisation method is called
	 * using a KeyPairGenerator interface, algorithm-specific defaults will
	 * be used. This will generate a new key pair every time it is called.
	 */
	public abstract KeyPair generateKeyPair();
}
