/*
 * $Id: Certificate.java,v 1.1 1999/01/19 05:14:59 wslade Exp $ 
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately.
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/cert/Certificate.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/19 05:14:59 $
 * $State: Exp $
 */

package au.net.aba.security.cert;

import java.security.*;
public abstract class Certificate 
{
	private String type;

	/*
	 * Constructors
	 */
	protected Certificate(String type) 
	{
		this.type = type;
	}

	/*
	 * Methods
	 */
	public boolean equals(Object other) 
	{
		// simple checks first.
		if (!(other instanceof Certificate))
		{
			return false;
		}

		if (this == other)
		{
			return true;
		}

		// ok, now are the various bits the same ?
		try 
		{
			byte[] tc = getEncoded();
			byte[] oc = ((Certificate)other).getEncoded();

			if (tc.length != oc.length)
			{
				return false;
			}

			for (int i = 0; i < tc.length; i++)
			{
				if (tc[i] != oc[i])
				{
					return false;
				}
			}
			return true;
		} 
		catch (CertificateException e) 
		{
			return false;
		}
	}

    public abstract byte[] getEncoded()
        throws CertificateEncodingException;

    public abstract PublicKey getPublicKey();

	public final String getType() 
	{
		return type;
	}

    public int hashCode() 
	{
		int     rv = 0;

		try 
		{
			byte[] data = getEncoded();
			for (int i = 1; i < data.length; i++) 
			{
				rv += data[i] * (data.length-i);
			}
			return rv;
		} 
		catch (CertificateException e) 
		{
			return rv;
		}
	}

    public abstract String toString();

    public abstract void verify(PublicKey key)
        throws 
		CertificateException, 
        InvalidKeyException, 
		NoSuchAlgorithmException,
		NoSuchProviderException,
        SignatureException;

    public abstract void verify(PublicKey key, String sigProvider)
        throws 
		CertificateException, 
        InvalidKeyException, 
		NoSuchAlgorithmException,
		NoSuchProviderException,
        SignatureException;

}
