/*
 * $Id: X509CRL.java,v 1.1 1999/01/19 05:15:02 wslade Exp $ 
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately.
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/cert/X509CRL.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/19 05:15:02 $
 * $State: Exp $
 */

package au.net.aba.security.cert;

import java.util.Date;
import java.math.BigInteger;
import au.net.aba.util.Set;
import java.security.*;

public abstract class X509CRL extends CRL implements X509Extension 
{

	protected X509CRL() 
	{
		super("X.509");
	}

	public boolean equals(Object other) 
	{
		// simple checks first.
		if (!(other instanceof Certificate))
		{
			return false;
		}

		if (this == other)
		{
			return true;
		}

		// ok, now are the various bits the same ?
		try 
		{
			byte[] tc = getEncoded();
			byte[] oc = ((X509CRL)other).getEncoded();

			if (tc.length != oc.length)
			{
				return false;
			}

			for (int i = 0; i < tc.length; i++)
			{
				if (tc[i] != oc[i])
				{
				return false;
				}
			}
			return true;
		} 
		catch (CRLException e) 
		{
			return false;
		}
	}

	public abstract byte[] getEncoded() throws CRLException;

	public abstract Principal getIssuerDN();

	public abstract Date getNextUpdate();

    public abstract X509CRLEntry getRevokedCertificate(BigInteger serialNumber);
  
	public abstract Set getRevokedCertificates();

	public abstract String getSigAlgName();

	public abstract String getSigAlgOID();

	public abstract byte[] getSigAlgParams();

	public abstract byte[] getSignature();

	public abstract byte[] getTBSCertList() throws CRLException;

	public abstract Date getThisUpdate();

	public abstract int getVersion();

	public int hashCode() 
	{
		int     rv = 0;

		try 
		{
			byte[] data = getEncoded();
			for (int i = 1; i < data.length; i++) 
			{
				rv += data[i] * (data.length-i);
			}
			return rv;
		} 
		catch (CRLException e) 
		{
			return rv;
		}
	}

	public abstract void verify(PublicKey key)
	throws CRLException,  
	InvalidKeyException, 
	NoSuchAlgorithmException,
	NoSuchProviderException,
	SignatureException;

	public abstract void verify(PublicKey key, String sigProvider)
	throws CRLException,  
	InvalidKeyException, 
	NoSuchAlgorithmException,
	NoSuchProviderException,
	SignatureException;
}
