/*
 * $Id: X509CRLEntry.java,v 1.1 1999/01/19 05:15:03 wslade Exp $ 
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately.
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/cert/X509CRLEntry.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/19 05:15:03 $
 * $State: Exp $
 */

package au.net.aba.security.cert;

import java.util.Date;
import java.math.BigInteger;

public abstract class X509CRLEntry implements X509Extension 
{

	public boolean equals(Object other) 
	{
		// simple checks first.
		if (!(other instanceof Certificate))
		{
			return false;
		}

		if (this == other)
		{
			return true;
		}

		// ok, now are the various bits the same ?
		try 
		{
			byte[] tc = getEncoded();
			byte[] oc = ((X509CRLEntry)other).getEncoded();

			if (tc.length != oc.length)
			{
				return false;
			}

			for (int i = 0; i < tc.length; i++)
			{
				if (tc[i] != oc[i])
				{
				return false;
				}
			}
			return true;
		} 
		catch (CRLException e) 
		{
			return false;
		}
	}

    public abstract byte[] getEncoded() throws CRLException;

    public abstract Date getRevocationDate();

    public abstract BigInteger getSerialNumber();

    public abstract boolean hasExtensions();

	public int hashCode() 
	{
		int     rv = 0;

		try 
		{
			byte[] data = getEncoded();
			for (int i = 1; i < data.length; i++) 
			{
				rv += data[i] * (data.length-i);
			}
			return rv;
		} 
		catch (CRLException e) 
		{
			return rv;
		}
	}

    public abstract String toString();
}
