/*
 * $Id: RSAPrivateCrtKey.java,v 1.1 1998/10/05 04:56:21 dgh Exp $
 * $Author: dgh $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/interfaces/RSAPrivateCrtKey.java,v $
 * $Revision: 1.1 $
 * $Date: 1998/10/05 04:56:21 $
 * $State: Exp $
 */

package au.net.aba.security.interfaces;

import java.math.*;
import java.security.*;

/**
 * The interface to an RSA private key, using the Chinese
 * Remainder Theorem (CRT) representation. 
 *
 * @see RSAPrivateKey
 */
public interface RSAPrivateCrtKey
	extends RSAPrivateKey
{
	public final static String ident = "$Id: RSAPrivateCrtKey.java,v 1.1 1998/10/05 04:56:21 dgh Exp $";

	/**
	 * Returns the public exponent.
	 *
	 * @return the public exponent.
	 */
	public BigInteger getPublicExponent();

	/**
	 * Returns the prime P.
	 *
	 * @return the prime P.
	 */
	public BigInteger getPrimeP();

	/**
	 * Returns the prime Q.
	 *
	 * @return the prime Q.
	 */
	public BigInteger getPrimeQ();

	/**
	 * Returns the primeExponentP.
	 *
	 * @return the primeExponentP.
	 */
	public BigInteger getPrimeExponentP();

	/**
	 * Returns the primeExponentQ.
	 *
	 * @return the primeExponentQ.
	 */
	public BigInteger getPrimeExponentQ();

	/**
	 * Returns the crtCoefficient.
	 *
	 * @return the crtCoefficient.
	 */
	public BigInteger getCrtCoefficient();
}
