/*
 * $Id: RSAPrivateKeySpec.java,v 1.3 1998/10/05 04:56:55 dgh Exp $
 * $Author: dgh $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/spec/RSAPrivateKeySpec.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/05 04:56:55 $
 * $State: Exp $
 */
package au.net.aba.security.spec;

import java.math.BigInteger;

/**
 * This class specifies an RSA private key. 
 *
 * @see Key
 * @see KeyFactory
 * @see KeySpec
 * @see PKCS8EncodedKeySpec
 * @see RSAPublicKeySpec
 * @see RSAPrivateKeyCrtSpec
 */
public class RSAPrivateKeySpec
	implements KeySpec
{
	public final static String ident = "$Id: RSAPrivateKeySpec.java,v 1.3 1998/10/05 04:56:55 dgh Exp $";

	private BigInteger	modulus;		// the modulus
	private BigInteger	privateExponent;	// the private exponent

	/**
	 * Basic constructor.
	 *
	 * @param modulus the modulus.
	 * @param privateExponent the private exponent.
	 */
	public RSAPrivateKeySpec(
		BigInteger	modulus,
		BigInteger	privateExponent)
	{
		this.modulus = modulus;
		this.privateExponent = privateExponent;
	}

	/**
	 * Returns the modulus.
	 *
	 * @return the modulus.
	 */
	public BigInteger getModulus()
	{
		return modulus;
	}

	/**
	 * Returns the public exponent.
	 *
	 * @return the public exponent.
	 */
	public BigInteger getPrivateExponent()
	{
		return privateExponent;
	}
}
