/*
 * $Id: DES.java,v 1.15 1999/01/18 05:01:39 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/DES.java,v $
 * $Revision: 1.15 $
 * $Date: 1999/01/18 05:01:39 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import java.security.*;

import java.security.*;
import java.security.spec.*;
import au.net.aba.crypto.spec.*;

import javax.crypto.*;
import javax.crypto.spec.*;

/**
 * A class that provides DES encryption.
 */
public class DES extends BlockCipher
{
	public final static String ident = "$Id: DES.java,v 1.15 1999/01/18 05:01:39 leachbj Exp $";

	protected int[] work;	/* working buffer */
	protected int[] Kn1;

	//==================================
	// Constructors
	//==================================

	public DES()
	{
		work = new int[2];
		Kn1 = new int[32];
	}

	protected void setKey(Key key)
	throws InvalidKeyException
	{
		if (!((key instanceof DESKey)
			|| (key instanceof SecretKeySpec)))
		{
			throw new InvalidKeyException("not a DES Key");
		}

		prepareKeys(mode, key.getEncoded(), Kn1);
	}

	protected int encryptBlock(
		byte[] in,
		int inOff,
		int len,
		byte[] out,
		int outOff)
	throws IllegalBlockSizeException
	{
		if (len != BLOCK_SIZE)
		{
			throw new IllegalBlockSizeException("Datasize less than block size.");
		}

		scrunch(in, inOff, work);
		desfunc(work, Kn1);
		unscrunch(work, out, outOff);

		return BLOCK_SIZE;
	}

	protected int decryptBlock(
		byte[] in,
		int inOff,
		int len,
		byte[] out,
		int outOff)
	throws BadPaddingException
	{
		if (len != BLOCK_SIZE)
		{
			throw new BadPaddingException("Datasize less than block size.");
		}

		scrunch(in, inOff, work);
		desfunc(work, Kn1);
		unscrunch(work, out, outOff);

		return BLOCK_SIZE;
	}

	/**
	 * what follows is largely taken from "Applied Cryptography", by
	 * Bruce Schneier. It also bears great resemblance to Richard
	 * Outerbridge's D3DES...
	 */

	static short[]	Df_Key =
		{
			0x01,0x23,0x45,0x67,0x89,0xab,0xcd,0xef,
			0xfe,0xdc,0xba,0x98,0x76,0x54,0x32,0x10,
			0x89,0xab,0xcd,0xef,0x01,0x23,0x45,0x67
		};

	static short[]	bytebit =
		{
			0200, 0100, 040, 020, 010, 04, 02, 01
		};

	static int[]	bigbyte =
		{
			0x800000, 0x400000, 0x200000, 0x100000,
			0x80000,  0x40000,  0x20000,  0x10000,
			0x8000,	  0x4000,   0x2000,   0x1000,
			0x800,    0x400,    0x200,    0x100,
			0x80,	  0x40,	    0x20,     0x10,
			0x8,      0x4,      0x2,      0x1
		};

	/* Use the key schedule specified in the Standard (ANSI X3.92-1981). */

	static byte[]	pc1 =
		{
			56, 48, 40, 32, 24, 16,  8,   0, 57, 49, 41, 33, 25, 17,
			 9,  1, 58, 50, 42, 34, 26,  18, 10,  2, 59, 51, 43, 35,
			62, 54, 46, 38, 30, 22, 14,   6, 61, 53, 45, 37, 29, 21,
			13,  5, 60, 52, 44, 36, 28,  20, 12,  4, 27, 19, 11,  3
		};

	static char[] totrot =
		{
			1, 2, 4, 6, 8, 10, 12, 14,
			15, 17, 19, 21, 23, 25, 27, 28
		};

	static char[] pc2 =
		{
			13, 16, 10, 23,  0,  4,  2, 27, 14,  5, 20,  9,
			22, 18, 11,  3, 25,  7, 15,  6, 26, 19, 12,  1,
			40, 51, 30, 36, 46, 54, 29, 39, 50, 44, 32, 47,
			43, 48, 38, 55, 33, 52, 45, 41, 49, 35, 28, 31
		};

	static int[] SP1 = {
		0x01010400, 0x00000000, 0x00010000, 0x01010404,
		0x01010004, 0x00010404, 0x00000004, 0x00010000,
		0x00000400, 0x01010400, 0x01010404, 0x00000400,
		0x01000404, 0x01010004, 0x01000000, 0x00000004,
		0x00000404, 0x01000400, 0x01000400, 0x00010400,
		0x00010400, 0x01010000, 0x01010000, 0x01000404,
		0x00010004, 0x01000004, 0x01000004, 0x00010004,
		0x00000000, 0x00000404, 0x00010404, 0x01000000,
		0x00010000, 0x01010404, 0x00000004, 0x01010000,
		0x01010400, 0x01000000, 0x01000000, 0x00000400,
		0x01010004, 0x00010000, 0x00010400, 0x01000004,
		0x00000400, 0x00000004, 0x01000404, 0x00010404,
		0x01010404, 0x00010004, 0x01010000, 0x01000404,
		0x01000004, 0x00000404, 0x00010404, 0x01010400,
		0x00000404, 0x01000400, 0x01000400, 0x00000000,
		0x00010004, 0x00010400, 0x00000000, 0x01010004
	};

	static int[] SP2 = {
		0x80108020, 0x80008000, 0x00008000, 0x00108020,
		0x00100000, 0x00000020, 0x80100020, 0x80008020,
		0x80000020, 0x80108020, 0x80108000, 0x80000000,
		0x80008000, 0x00100000, 0x00000020, 0x80100020,
		0x00108000, 0x00100020, 0x80008020, 0x00000000,
		0x80000000, 0x00008000, 0x00108020, 0x80100000,
		0x00100020, 0x80000020, 0x00000000, 0x00108000,
		0x00008020, 0x80108000, 0x80100000, 0x00008020,
		0x00000000, 0x00108020, 0x80100020, 0x00100000,
		0x80008020, 0x80100000, 0x80108000, 0x00008000,
		0x80100000, 0x80008000, 0x00000020, 0x80108020,
		0x00108020, 0x00000020, 0x00008000, 0x80000000,
		0x00008020, 0x80108000, 0x00100000, 0x80000020,
		0x00100020, 0x80008020, 0x80000020, 0x00100020,
		0x00108000, 0x00000000, 0x80008000, 0x00008020,
		0x80000000, 0x80100020, 0x80108020, 0x00108000
	};

	static int[] SP3 = {
		0x00000208, 0x08020200, 0x00000000, 0x08020008,
		0x08000200, 0x00000000, 0x00020208, 0x08000200,
		0x00020008, 0x08000008, 0x08000008, 0x00020000,
		0x08020208, 0x00020008, 0x08020000, 0x00000208,
		0x08000000, 0x00000008, 0x08020200, 0x00000200,
		0x00020200, 0x08020000, 0x08020008, 0x00020208,
		0x08000208, 0x00020200, 0x00020000, 0x08000208,
		0x00000008, 0x08020208, 0x00000200, 0x08000000,
		0x08020200, 0x08000000, 0x00020008, 0x00000208,
		0x00020000, 0x08020200, 0x08000200, 0x00000000,
		0x00000200, 0x00020008, 0x08020208, 0x08000200,
		0x08000008, 0x00000200, 0x00000000, 0x08020008,
		0x08000208, 0x00020000, 0x08000000, 0x08020208,
		0x00000008, 0x00020208, 0x00020200, 0x08000008,
		0x08020000, 0x08000208, 0x00000208, 0x08020000,
		0x00020208, 0x00000008, 0x08020008, 0x00020200
	};

	static int[] SP4 = {
		0x00802001, 0x00002081, 0x00002081, 0x00000080,
		0x00802080, 0x00800081, 0x00800001, 0x00002001,
		0x00000000, 0x00802000, 0x00802000, 0x00802081,
		0x00000081, 0x00000000, 0x00800080, 0x00800001,
		0x00000001, 0x00002000, 0x00800000, 0x00802001,
		0x00000080, 0x00800000, 0x00002001, 0x00002080,
		0x00800081, 0x00000001, 0x00002080, 0x00800080,
		0x00002000, 0x00802080, 0x00802081, 0x00000081,
		0x00800080, 0x00800001, 0x00802000, 0x00802081,
		0x00000081, 0x00000000, 0x00000000, 0x00802000,
		0x00002080, 0x00800080, 0x00800081, 0x00000001,
		0x00802001, 0x00002081, 0x00002081, 0x00000080,
		0x00802081, 0x00000081, 0x00000001, 0x00002000,
		0x00800001, 0x00002001, 0x00802080, 0x00800081,
		0x00002001, 0x00002080, 0x00800000, 0x00802001,
		0x00000080, 0x00800000, 0x00002000, 0x00802080
	};

	static int[] SP5 = {
		0x00000100, 0x02080100, 0x02080000, 0x42000100,
		0x00080000, 0x00000100, 0x40000000, 0x02080000,
		0x40080100, 0x00080000, 0x02000100, 0x40080100,
		0x42000100, 0x42080000, 0x00080100, 0x40000000,
		0x02000000, 0x40080000, 0x40080000, 0x00000000,
		0x40000100, 0x42080100, 0x42080100, 0x02000100,
		0x42080000, 0x40000100, 0x00000000, 0x42000000,
		0x02080100, 0x02000000, 0x42000000, 0x00080100,
		0x00080000, 0x42000100, 0x00000100, 0x02000000,
		0x40000000, 0x02080000, 0x42000100, 0x40080100,
		0x02000100, 0x40000000, 0x42080000, 0x02080100,
		0x40080100, 0x00000100, 0x02000000, 0x42080000,
		0x42080100, 0x00080100, 0x42000000, 0x42080100,
		0x02080000, 0x00000000, 0x40080000, 0x42000000,
		0x00080100, 0x02000100, 0x40000100, 0x00080000,
		0x00000000, 0x40080000, 0x02080100, 0x40000100
	};

	static int[] SP6 = {
		0x20000010, 0x20400000, 0x00004000, 0x20404010,
		0x20400000, 0x00000010, 0x20404010, 0x00400000,
		0x20004000, 0x00404010, 0x00400000, 0x20000010,
		0x00400010, 0x20004000, 0x20000000, 0x00004010,
		0x00000000, 0x00400010, 0x20004010, 0x00004000,
		0x00404000, 0x20004010, 0x00000010, 0x20400010,
		0x20400010, 0x00000000, 0x00404010, 0x20404000,
		0x00004010, 0x00404000, 0x20404000, 0x20000000,
		0x20004000, 0x00000010, 0x20400010, 0x00404000,
		0x20404010, 0x00400000, 0x00004010, 0x20000010,
		0x00400000, 0x20004000, 0x20000000, 0x00004010,
		0x20000010, 0x20404010, 0x00404000, 0x20400000,
		0x00404010, 0x20404000, 0x00000000, 0x20400010,
		0x00000010, 0x00004000, 0x20400000, 0x00404010,
		0x00004000, 0x00400010, 0x20004010, 0x00000000,
		0x20404000, 0x20000000, 0x00400010, 0x20004010
	};

	static int[] SP7 = {
		0x00200000, 0x04200002, 0x04000802, 0x00000000,
		0x00000800, 0x04000802, 0x00200802, 0x04200800,
		0x04200802, 0x00200000, 0x00000000, 0x04000002,
		0x00000002, 0x04000000, 0x04200002, 0x00000802,
		0x04000800, 0x00200802, 0x00200002, 0x04000800,
		0x04000002, 0x04200000, 0x04200800, 0x00200002,
		0x04200000, 0x00000800, 0x00000802, 0x04200802,
		0x00200800, 0x00000002, 0x04000000, 0x00200800,
		0x04000000, 0x00200800, 0x00200000, 0x04000802,
		0x04000802, 0x04200002, 0x04200002, 0x00000002,
		0x00200002, 0x04000000, 0x04000800, 0x00200000,
		0x04200800, 0x00000802, 0x00200802, 0x04200800,
		0x00000802, 0x04000002, 0x04200802, 0x04200000,
		0x00200800, 0x00000000, 0x00000002, 0x04200802,
		0x00000000, 0x00200802, 0x04200000, 0x00000800,
		0x04000002, 0x04000800, 0x00000800, 0x00200002
	};

	static int[] SP8 = {
		0x10001040, 0x00001000, 0x00040000, 0x10041040,
		0x10000000, 0x10001040, 0x00000040, 0x10000000,
		0x00040040, 0x10040000, 0x10041040, 0x00041000,
		0x10041000, 0x00041040, 0x00001000, 0x00000040,
		0x10040000, 0x10000040, 0x10001000, 0x00001040,
		0x00041000, 0x00040040, 0x10040040, 0x10041000,
		0x00001040, 0x00000000, 0x00000000, 0x10040040,
		0x10000040, 0x10001000, 0x00041040, 0x00040000,
		0x00041040, 0x00040000, 0x10041000, 0x00001000,
		0x00000040, 0x10040040, 0x00001000, 0x00041040,
		0x10001000, 0x00000040, 0x10000040, 0x10040000,
		0x10040040, 0x10000000, 0x00040000, 0x10001040,
		0x00000000, 0x10041040, 0x00040040, 0x10000040,
		0x10040000, 0x10001000, 0x10001040, 0x00000000,
		0x10041040, 0x00041000, 0x00041000, 0x00001040,
		0x00001040, 0x00040040, 0x10000000, 0x10041000
	};

	/**
	 * prepare the key for whatever processing we are planing.
	 *
	 * Acknowledgements for this routine go to James Gillogly & Phil Karn.
	 * 		(whoever, and wherever they are!).
	 */
	protected void prepareKeys(
		int	opMode,
		byte[]	key,
		int[]	keys)
	{
		byte[]	pc1m, pcr;
		int[]	kn;

		pc1m = new byte[56];
		pcr = new byte[56];
		kn = new int[32];

		for (int j = 0; j < 56; j++ )
		{
			int	l, m;

			l = pc1[j];
			m = l & 07;
			pc1m[j] = (byte)(((key[l >>> 3] & bytebit[m]) != 0)
								? 1 : 0);
		}

		for (int i = 0; i < 16; i++)
		{
			int	l, m, n;

			if (opMode == Cipher.DECRYPT_MODE)
			{
				m = (15 - i) << 1;
			}
			else
			{
				m = i << 1;
			}

			n = m + 1;
			kn[m] = kn[n] = 0;

			for (int j = 0; j < 28; j++)
			{
				l = j + totrot[i];
				if ( l < 28 )
				{
					pcr[j] = pc1m[l];
				}
				else
				{
					pcr[j] = pc1m[l - 28];
				}
			}

			for (int j = 28; j < 56; j++)
			{
				l = j + totrot[i];
				if (l < 56 )
				{
					pcr[j] = pc1m[l];
				}
				else
				{
					pcr[j] = pc1m[l - 28];
				}
			}

			for (int j = 0; j < 24; j++)
			{
				if (pcr[pc2[j]] != 0)
				{
					kn[m] |= bigbyte[j];
				}

				if (pcr[pc2[j + 24]] != 0)
				{
					kn[n] |= bigbyte[j];
				}
			}
		}

		/*
		 * store the processed key
		 */
		for (int i = 0; i != 16; i++)
		{
			int	i1, i2;

			i1 = kn[2 * i];
			i2 = kn[2 * i + 1];

			keys[i * 2] = (i1 & 0x00fc0000) << 6;
			keys[i * 2] |= (i1 & 0x00000fc0) << 10;
			keys[i * 2] |= (i2 & 0x00fc0000) >>> 10;
			keys[i * 2] |= (i2 & 0x00000fc0) >>> 6;

			keys[i * 2 + 1] = (i1 & 0x0003f000) << 12;
			keys[i * 2 + 1] |= (i1 & 0x0000003f) << 16;
			keys[i * 2 + 1] |= (i2 & 0x0003f000) >>> 4;
			keys[i * 2 + 1] |= (i2 & 0x0000003f);
		}
	}

	/**
	 * pack 8 bytes from outof into 2 ints.
	 */
	protected void scrunch(
		byte[]	outof,
		int	offset,
		int[]	into)
	{
		into[0]	 = (outof[0 + offset] & 0xff) << 24;
		into[0]	|= (outof[1 + offset] & 0xff) << 16;
		into[0]	|= (outof[2 + offset] & 0xff) << 8;
		into[0] |= (outof[3 + offset] & 0xff);
		into[1]	 = (outof[4 + offset] & 0xff) << 24;
		into[1]	|= (outof[5 + offset] & 0xff) << 16;
		into[1]	|= (outof[6 + offset] & 0xff) << 8;
		into[1]	|= (outof[7 + offset] & 0xff);
	}

	/**
	 * extract 8 bytes from the 2 ints in outof.
	 */
	protected void unscrunch(
		int[]	outof,
		byte[]	into,
		int	offset)
	{
		into[0 + offset] = (byte)((outof[0] >>> 24) & 0xff);
		into[1 + offset] = (byte)((outof[0] >>> 16) & 0xff);
		into[2 + offset] = (byte)((outof[0] >>>  8) & 0xff);
		into[3 + offset] = (byte)( outof[0]	    & 0xff);
		into[4 + offset] = (byte)((outof[1] >>> 24) & 0xff);
		into[5 + offset] = (byte)((outof[1] >>> 16) & 0xff);
		into[6 + offset] = (byte)((outof[1] >>>  8) & 0xff);
		into[7 + offset] = (byte)( outof[1]	    & 0xff);
	}

	/**
	 * the DES engine.
	 */
	protected synchronized void desfunc(
		int[]	block,
		int[]	keys)
	{
		int	fval, work, right, leftt;
		int	round;

		leftt = block[0];
		right = block[1];
		work = ((leftt >>> 4) ^ right) & 0x0f0f0f0f;
		right ^= work;
		leftt ^= (work << 4);
		work = ((leftt >>> 16) ^ right) & 0x0000ffff;
		right ^= work;
		leftt ^= (work << 16);
		work = ((right >>> 2) ^ leftt) & 0x33333333;
		leftt ^= work;
		right ^= (work << 2);
		work = ((right >>> 8) ^ leftt) & 0x00ff00ff;
		leftt ^= work;
		right ^= (work << 8);
		right = ((right << 1) | ((right >>> 31) & 1)) & 0xffffffff;
		work = (leftt ^ right) & 0xaaaaaaaa;
		leftt ^= work;
		right ^= work;
		leftt = ((leftt << 1) | ((leftt >>> 31) & 1)) & 0xffffffff;

		for (round = 0; round < 8; round++)
		{
			work  = (right << 28) | (right >>> 4);
			work ^= keys[round * 4 + 0];
			fval  = SP7[ work	  & 0x3f];
			fval |= SP5[(work >>>  8) & 0x3f];
			fval |= SP3[(work >>> 16) & 0x3f];
			fval |= SP1[(work >>> 24) & 0x3f];
			work  = right ^ keys[round * 4 + 1];
			fval |= SP8[ work	  & 0x3f];
			fval |= SP6[(work >>>  8) & 0x3f];
			fval |= SP4[(work >>> 16) & 0x3f];
			fval |= SP2[(work >>> 24) & 0x3f];
			leftt ^= fval;
			work  = (leftt << 28) | (leftt >>> 4);
			work ^= keys[round * 4 + 2];
			fval  = SP7[ work	  & 0x3f];
			fval |= SP5[(work >>>  8) & 0x3f];
			fval |= SP3[(work >>> 16) & 0x3f];
			fval |= SP1[(work >>> 24) & 0x3f];
			work  = leftt ^ keys[round * 4 + 3];
			fval |= SP8[ work	  & 0x3f];
			fval |= SP6[(work >>>  8) & 0x3f];
			fval |= SP4[(work >>> 16) & 0x3f];
			fval |= SP2[(work >>> 24) & 0x3f];
			right ^= fval;
		}

		right = (right << 31) | (right >>> 1);
		work = (leftt ^ right) & 0xaaaaaaaa;
		leftt ^= work;
		right ^= work;
		leftt = (leftt << 31) | (leftt >>> 1);
		work = ((leftt >>> 8) ^ right) & 0x00ff00ff;
		right ^= work;
		leftt ^= (work << 8);
		work = ((leftt >>> 2) ^ right) & 0x33333333;
		right ^= work;
		leftt ^= (work << 2);
		work = ((right >>> 16) ^ leftt) & 0x0000ffff;
		leftt ^= work;
		right ^= (work << 16);
		work = ((right >>> 4) ^ leftt) & 0x0f0f0f0f;
		leftt ^= work;
		right ^= (work << 4);

		block[0] = right;
		block[1] = leftt;
	}
}
