/*
 * $Id: DESedeKeyGenerator.java,v 1.5 1999/01/13 07:31:53 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/DESedeKeyGenerator.java,v $
 * $Revision: 1.5 $
 * $Date: 1999/01/13 07:31:53 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.security.*;

/**
 * This class is used for generating random DESede keys.  This class
 * should not be instantiated directly, instead use the
 * javax.crypto.KeyGenerator interface.
 * <p>
 * There is no AlgorithmParameterSpec class defined for DESede so this
 * generator can only be initialised using the keysize,random
 * initialisation.
 * <p>
 * The returned key will be a non-weak key with odd parity.
 */
public class DESedeKeyGenerator extends DESKeyGenerator
{
	public final static String ident = "$Id: DESedeKeyGenerator.java,v 1.5 1999/01/13 07:31:53 leachbj Exp $";

	/**
	 * Generates a secret key, setting odd parity and checking that the
	 * key is not a weak key.
	 *
	 * @return a DESede key object.
	 */
	protected SecretKey engineGenerateKey()
	{
		// generate random non-weak key
		int	count = 0;
		byte[]	keyBytes = new byte[DESedeKeySpec.DES_EDE_KEY_LEN];
		byte[]	bytes = new byte[DESKeySpec.DES_KEY_LEN];

		DESedeKey key = null;

		do {
			// get 64 bits of random data, 8 bits are wasted :(
			rand.nextBytes(bytes);

			// set the parity to odd
			setOddParity(bytes);

			// check for weak keys
			if (!isWeakKey(bytes))
			{
				System.arraycopy(bytes, 0, keyBytes,
						count++ * DESKeySpec.DES_KEY_LEN, DESKeySpec.DES_KEY_LEN);

				/*
				 * clear the key from the
				 * byte buffer
				 */
				for (int i = 0; i < DESKeySpec.DES_KEY_LEN; i++)
				{
					bytes[i] = 0;
				}

				if (count == 3)
				{
					key = new DESedeKey(keyBytes);

					/*
					 * clear the key from the
					 * byte buffer
					 */
					for (int i = 0;
						i < DESedeKeySpec.DES_EDE_KEY_LEN; i++)
					{
						keyBytes[i] = 0;
					}
				}
			}
		} while (key == null);

		return key;
	}
}
