/*
 * $Id: RC4KeyFactory.java,v 1.4 1998/10/27 05:10:38 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RC4KeyFactory.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/27 05:10:38 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import javax.crypto.*;
import au.net.aba.crypto.spec.RC4KeySpec;
import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * This class is used to convert RC4 keys into a format usable by the
 * ABA provider.  Currently this class can only convert from a KeySpec
 * into a Key.  The supported KeySpec class is RC4KeySpec.
 * <p>
 * This class should not be instantiated directly, instead use the
 * java.security.KeyFactory interface.
 *
 * @see KeyFactory
 */
public class RC4KeyFactory extends SecretKeyFactorySpi
{
	/**
	 * Generates a RC4 SecretKey object from the provided key specification
	 * (key material).  This class supports the RC4KeySpec KeySpec class.
	 *
	 * @returns	The secret key.
	 * @exception InvalidKeySpecException	The provided KeySpec was not
	 * 	a RC4KeySpec KeySpec.
	 */
	protected SecretKey engineGenerateSecret(
		KeySpec keySpec)
		throws InvalidKeySpecException
	{
		if (keySpec == null || !(keySpec instanceof RC4KeySpec))
		{
			throw new InvalidKeySpecException("Invalid KeySpec");
		}
		return new RC4Key(((RC4KeySpec)keySpec).getKey());
	}

	/**
	 * Returns a specification (key material) of the given key object in
	 * the requested format. 
	 * <p>
	 * Not currently implemented.
	 *
	 * @param key the key 
	 * @param keySpec the requested format in which the key material shall
	 *		be returned 
	 * @returns the underlying key specification (key material) in the
	 *		requested format
	 * @exception InvalidKeySpecException if the requested key
	 *		specification is inappropriate for the given key, or the given
	 *		key cannot be dealt with (e.g., the given key has an
	 *		unrecognised format). 
	 */
	protected KeySpec engineGetKeySpec(
		SecretKey key,
		Class keySpec)
		throws InvalidKeySpecException
	{
		throw new InvalidKeySpecException("not implemented");
	}

	/**
	 * Translates a RC4 key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this key
	 * factory. 
	 * <p>
	 * Not currently implemented.
	 *
	 * @param key - the key whose provider is unknown or untrusted 
	 * @returns the translated key 
	 * @exception InvalidKeyException if the given key cannot be processed
	 * 	by this key factory. 
	 */
	protected SecretKey engineTranslateKey(
		SecretKey key)
		throws InvalidKeyException
	{
		throw new InvalidKeyException("not implemented");
	}
}
