/*
 * $Id: RC4KeyGenerator.java,v 1.6 1999/01/13 07:28:51 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RC4KeyGenerator.java,v $
 * $Revision: 1.6 $
 * $Date: 1999/01/13 07:28:51 $
 * $State: Exp $
 */
package au.net.aba.crypto.provider;

import javax.crypto.*;
import javax.crypto.spec.*;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * This class is used for generating random RC4 keys.  This class
 * should not be instantiated directly, instead use the
 * javax.crypto.KeyGenerator interface.
 * <p>
 * There is no AlgorithmParameterSpec class defined for RC4 so this
 * generator can only be initialised using the keysize,random
 * initialisation.
 * <p>
 * The default keysize is 128 bits, and may be any multiple of 8 less
 * than (or equal to) 256.
 */
public class RC4KeyGenerator extends KeyGeneratorSpi
{
	public final static String ident = "$Id: RC4KeyGenerator.java,v 1.6 1999/01/13 07:28:51 leachbj Exp $";

	static final int	DEF_KEY_SIZE = 16;	// standard key size (bytes)

	private SecureRandom		rand;

	/**
	 * Initialises the key generator with the specified parameter set and
	 * a user-provided source of randomness.
	 */
	protected void engineInit(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
	{
		rand = random;
	}

	/**
	 * Initialises the key generator with the given random number source.
	 */
	protected void engineInit(
		SecureRandom	random)
	{
		rand = random;
	}

	/**
	 * Generates a random secret key.
	 *
	 * @return the RC4 key.
	 */
	protected SecretKey engineGenerateKey()
	{
		byte[]		bytes;

		/*
		 * In case engineInit() was not called
		 */
		if (rand == null)
		{
			rand = new SecureRandom();
		}

		bytes = new byte[DEF_KEY_SIZE];
		rand.nextBytes(bytes);

		return new RC4Key(bytes);
	}
}
