/*
 * $Id: TwofishKeyGenerator.java,v 1.4 1998/10/27 05:11:06 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.         
 *       
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/TwofishKeyGenerator.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/27 05:11:06 $
 * $State: Exp $
 */
package au.net.aba.crypto.provider;

import java.security.InvalidAlgorithmParameterException;
import java.security.spec.AlgorithmParameterSpec;
import java.security.SecureRandom;
import javax.crypto.KeyGeneratorSpi;
import javax.crypto.SecretKey;

/**
 * This class is used for generating random Twofish keys.  This class
 * should not be instantiated directly, instead use the
 * javax.crypto.KeyGenerator interface.
 * <p>
 * There is no AlgorithmParameterSpec class defined for Twofish so this
 * generator can only be initialised using the keysize,random
 * initialisation.
 * <p>
 * The default keysize is 128 bits, and may be up to 256 bits long.
 */
public class TwofishKeyGenerator extends KeyGeneratorSpi
{
	public final static String ident = "$Id: TwofishKeyGenerator.java,v 1.4 1998/10/27 05:11:06 leachbj Exp $";

	static final int	DEF_KEY_SIZE = 128;		/* standard key size in bits */

	private SecureRandom		rand;
	private int			strength = DEF_KEY_SIZE;

	/**
	 * This method is not implemented as there is no AlgorithmParameterSpec
	 * defined for Twofish.  (Use one of the other initialisation methods!)
	 *
	 * @param params	the algorithm parameter specs for this
	 *		generator.
	 * @param random	a source of random numbers for this generator.
	 * @exception InvalidAlgorithmParameterException	An invalid
	 * 	parameter specification is provided.
	 */
	protected void engineInit(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidAlgorithmParameterException
	{
		throw new InvalidAlgorithmParameterException("Not Implemented");
	}

	/**
	 * Initialises this key generator for a certain strength, using the
	 * given source of randomness.
	 *
	 * @param strength the strength of the key. This is an
	 *    algorithm-specific metric specified in number of bits.
	 * @param random the source of randomness for this key generator
	 */
	protected void engineInit(
			int             strength,
			SecureRandom    random)
	{
		this.strength = strength;
		this.rand = random;
	}

	/**
	 * Initialises the key generator with the given random source.
	 *
	 * @param random	a source of random numbers for this generator.
	 */
	protected void engineInit(
		SecureRandom	random)
	{
		rand = random;
	}

	/**
	 * Generates a random secret key.
	 *
	 * @return a secret key representing a Twofish key.
	 */
	protected SecretKey engineGenerateKey()
	{
		byte[]		bytes;

		/*
		 * In case engineInit() was not called
		 */
		if (rand == null)
		{
			rand = new SecureRandom();
		}

		bytes = new byte[strength / 8];
		rand.nextBytes(bytes);

		return new TwofishKey(bytes);
	}
}
