/*
 * $Id: AsciiEncodedKeySpec.java,v 1.4 1998/10/26 01:46:47 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/spec/AsciiEncodedKeySpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/26 01:46:47 $
 * $State: Exp $
 */

package au.net.aba.crypto.spec;

import au.net.aba.security.spec.EncodedKeySpec;

/**
 * A key in an ASCII encoded format. ASCII encoded key specs are expected to
 * be an array of UTF8 characters representing a hexadecimal number, or a
 * set of hexadecimal numbers. Specs made of more than 1 number, such as an
 * RSAPrivateKey have the numbers separated by '.' characters, and the numbers
 * are given in the same order as the equivalent key spec class; e.g an
 * RSAPrivate key is the string "modulus.privateExponent" where modulus
 * and privateExponent are both the hex representations of big numbers.
 */
public class AsciiEncodedKeySpec
	extends EncodedKeySpec
{
	public final static String ident = "$Id: AsciiEncodedKeySpec.java,v 1.4 1998/10/26 01:46:47 leachbj Exp $";

	/**
	 * Create the KeySpec from the given string.
	 */
	public AsciiEncodedKeySpec(String key)
	{
		super(key.getBytes());
	}

	/**
	 * Returns the name of the encoding format used by this
	 * key specification.
	 *
	 * @return the string "ASCII"
	 */
	public String getFormat()
	{
		return "ASCII";
	}
}
