/*
 * $Id: InlineIvParameterSpec.java,v 1.5 1998/10/28 23:20:47 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/spec/InlineIvParameterSpec.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/28 23:20:47 $
 * $State: Exp $
 */

package au.net.aba.crypto.spec;

import java.security.spec.*;

/**
 * This class specifies that an initialisation vector (IV) is contained in
 * the input stream/output stream produced by the cipher.
 *
 * @see javax.crypto.spec.IvParameterSpec
 * @see java.security.spec.AlgorithmParameterSpec
 */
public class InlineIvParameterSpec implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: InlineIvParameterSpec.java,v 1.5 1998/10/28 23:20:47 leachbj Exp $";

	private boolean	encrypted;

	/**
	 * Specify that the IV is not encrypted.
	 */
	public InlineIvParameterSpec()
	{
		encrypted = false;
	}

	/**
	 * Specify whether or not the IV is encrypted.
	 *
	 * @param encrypted true if it is, false otherwise.
	 */
	public InlineIvParameterSpec(
		boolean	encrypted)
	{
		this.encrypted = encrypted;
	}

	/**
	 * Returns whether we can expect an encrypted IV.
	 *
	 * @return true if the IV is encrypted, false otherwise.
	 */
	public boolean isEncryptedIv()
	{
		return encrypted;
	}
}
