/*
 * $Id: RC4KeySpec.java,v 1.2 1998/10/20 04:02:56 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/spec/RC4KeySpec.java,v $
 * $Revision: 1.2 $
 * $Date: 1998/10/20 04:02:56 $
 * $State: Exp $
 */

package au.net.aba.crypto.spec;

import javax.crypto.*;
import au.net.aba.security.*;
import au.net.aba.security.spec.KeySpec;

/**
 * A class that provides a specification for a RC4 key.
 */
public class RC4KeySpec implements KeySpec
{
	public final static String ident = "$Id: RC4KeySpec.java,v 1.2 1998/10/20 04:02:56 leachbj Exp $";

	private byte[]	rc4Key;

	/**
	 * Uses the first 16 bytes in key as the RC4 key 
	 */
	public RC4KeySpec(
		byte[]	key)
		throws InvalidKeyException
	{
		this(key, 0, 16);
	}

	/**
	 * Uses the first len bytes in key as the RC4 key 
	 */
	public RC4KeySpec(
		byte[]	key,
		int	len)
		throws InvalidKeyException
	{
		this(key, 0, len);
	}

	/**
	 * Uses the first 16 bytes in key, beginning at offset, as the RC4 key
	 */
	public RC4KeySpec(
		byte[]	key,
		int	offset,
		int	len)
		throws InvalidKeyException
	{
		if (key.length - offset < len)
		{
			throw new InvalidKeyException("Key too short");
		}
		rc4Key = new byte[len];
		System.arraycopy(key, offset, rc4Key, 0, len);
	}

	/**
	 * Returns the RC4 key.
	 */
	public byte[] getKey()
	{
		return rc4Key;
	}
}
