/*
 * $Id: ByteUtil.java,v 1.2 1998/10/20 07:49:43 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/ByteUtil.java,v $
 * $Revision: 1.2 $
 * $Date: 1998/10/20 07:49:43 $
 * $State: Exp $
 */

package aba.test;

import java.text.*;

/**
 * A utility class for processing arrays of bytes.
 */
public class ByteUtil
{
	public final static String ident = "$Id: ByteUtil.java,v 1.2 1998/10/20 07:49:43 leachbj Exp $";

	/**
	 * Transform the specified byte into a Hex String form.
	 *
	 * @param bcd		The byte to transform.
	 * @return		The Hex String.
	 */
	public static final String bytesToHexStr(
		byte[] bcd)
	{
		StringBuffer s = new StringBuffer(bcd.length * 2);

		for (int i = 0; i < bcd.length; i++)
		{
			s.append(bcdLookup[(bcd[i] >>> 4) & 0x0f]);
			s.append(bcdLookup[bcd[i] & 0x0f]);
		}

		return s.toString();
	}

	/**
	 * print out a hex block, 20 bytes per line, with numbering down
	 * one side.
	 */
	public static void printHexBlock(
		String s,
		byte[] b)
	{
		String		out;
		DecimalFormat	format;

		out = bytesToHexStr(b);  
		format = new DecimalFormat("00000");

		System.out.println(s + " - bytes : " + b.length);

		for (int i = 0; i < out.length(); i += 40)
		{
			try
			{
				System.out.println(format.format(i / 2)
					+ " " + out.substring(i, i + 40));
			}
			catch (StringIndexOutOfBoundsException e)
			{
				System.out.println(format.format(i / 2)
					+ " " + out.substring(i));
			}
		}
	}

	/**
	 * Transform the specified Hex String into a byte array.
	 *
	 * @param s		the hex string to convert.		
	 * @return		The resulting byte array.
	 */
	public static final byte[] hexStrToBytes(
		String	s)
	{
		byte[]	bytes;

		bytes = new byte[s.length() / 2];

		for (int i = 0; i < bytes.length; i++)
		{
			bytes[i] = (byte)Integer.parseInt(
					s.substring(2 * i, 2 * i + 2), 16);
		}

		return bytes;
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * Lookup table for converting Hex byte values into chars
	 */
	private static final char[] bcdLookup =
	{
		'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'
	};
}

