/*
 * $Id: CBCVectorTest.java,v 1.1 1999/02/17 04:54:46 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/CBCVectorTest.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/02/17 04:54:46 $
 * $State: Exp $
 */

package aba.test;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.security.*;

import au.net.aba.crypto.provider.ABAProvider;

/**
 * Test code for checking a Cipher in CBC mode against known
 * test vectors.
 */
public class CBCVectorTest
{
	static Cipher			out;
	static String			algorithm;

	public static void main(
		String[]	args)
		throws Exception
	{
		ABAProvider		prov;
		BufferedReader		bRd;

		prov = new ABAProvider();
		Security.addProvider(prov);
		
		algorithm = args[0];

		bRd = new BufferedReader(
				new FileReader(args[2]));

		try
		{
			out = Cipher.getInstance(
					args[1], "ABA");
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);

			System.err.println(e);
			System.exit(2);
			return;
		}

		/*
		 * read and process the file.
		 */
		String	line;

		while ((line = bRd.readLine()) != null)
		{
			StringTokenizer	st;

			st = new StringTokenizer(line);
			process(st.nextToken(), st.nextToken(), st.nextToken(),
				st.nextToken());
		}

	}

	public static void process(
		String	keyStr,
		String	iv,
		String	plainText,
		String	cipherText)
		throws Exception
	{
		byte[]			enc;
		SecretKeySpec		key;
		String			encText;

		key = new SecretKeySpec(ByteUtil.hexStrToBytes(keyStr),
			algorithm);

		IvParameterSpec ivSpec = new IvParameterSpec(
			ByteUtil.hexStrToBytes(iv));
		out.init(Cipher.ENCRYPT_MODE, key, ivSpec);

		/*
		 * encipher
		 */
		enc = out.doFinal(ByteUtil.hexStrToBytes(plainText));

		cipherText = cipherText.toLowerCase();
		encText = ByteUtil.bytesToHexStr(enc).toLowerCase();

		System.out.println(keyStr.toLowerCase()
			+ ":" + plainText.toLowerCase()
			+ ":" + iv.toLowerCase()
			+ ":" + cipherText + ":"
			+ encText + ":" + cipherText.equals(encText));
	}
}
