/*
 * $Id: CipherTest.java,v 1.1 1999/02/19 03:51:32 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/test/CipherTest.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/02/19 03:51:32 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import au.net.aba.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.security.*;

/**
 * basic test class for a Cipher.
 */
public class CipherTest
{
	public static void main(
		String[]	args)
		throws KeyException, InvalidAlgorithmParameterException
	{
		Key			key;
		KeyGenerator		keyGen;
		SecureRandom		rand;
		Cipher			in, out;
		CipherInputStream	cIn;
		CipherOutputStream	cOut;
		ByteArrayInputStream	bIn;
		ByteArrayOutputStream	bOut;

		rand = new SecureRandom();

		try
		{
			keyGen = KeyGenerator.getInstance(args[0], "ABA");
			keyGen.init(rand);

			key = keyGen.generateKey();

			in = Cipher.getInstance(
					args[1], "ABA");
			out = Cipher.getInstance(
					args[1], "ABA");
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);

			System.err.println(e);
			System.exit(2);
			return;
		}

		out.init(Cipher.ENCRYPT_MODE, key);

		/*
		 * grab the iv if there is one
		 */
		byte[]	iv;

		iv = out.getIV();
		if (iv != null)
		{
			IvParameterSpec	spec;

			spec = new IvParameterSpec(iv);

			in.init(Cipher.DECRYPT_MODE, key, spec);
		}
		else
		{
			in.init(Cipher.DECRYPT_MODE, key);
		}

		/*
		 * encipher
		 */
		bOut = new ByteArrayOutputStream();

		cOut = new CipherOutputStream(bOut, out);

		String orig = args[0] + args[2];
		byte[] oBytes = new byte[orig.length()];
		orig.getBytes(0, orig.length(), oBytes, 0);

		System.out.println("orig: "
			+ ByteUtil.bytesToHexStr(oBytes));

		try
		{
			for (int i = 0; i != args[0].length(); i++)
			{
				cOut.write((byte)args[0].charAt(i));
			}
			byte[] tmp = new byte[args[2].length()];
			args[2].getBytes(0, args[2].length(), tmp, 0);
			cOut.write(tmp);
			cOut.close();
		}
		catch (IOException e)
		{
			e.printStackTrace(System.err);
		}

		/*
		 * decipher
		 */
		byte[]	bytes;

		bytes = bOut.toByteArray();

		System.out.println("enc:  " + ByteUtil.bytesToHexStr(bytes));

		bIn = new ByteArrayInputStream(bytes);

		cIn = new CipherInputStream(bIn, in);

		try
		{
			DataInputStream dIn = new DataInputStream(cIn);

			if (args[1].endsWith("NoPadding"))
			{
				bytes = new byte[bytes.length];
			}
			else
			{
				bytes = new byte[args[2].length()
							+ args[0].length()];
			}

			for (int i = 0; i != args[0].length(); i++)
			{
				bytes[i] = (byte)dIn.read();
			}
			dIn.readFully(bytes, args[0].length(),
					bytes.length - args[0].length());
		}
		catch (IOException e)
		{
			System.err.println(e);
			e.printStackTrace(System.err);
			System.out.println("error dec:  "
					+ ByteUtil.bytesToHexStr(bytes));
			System.exit(4);
		}

		System.out.println("dec:  " + ByteUtil.bytesToHexStr(bytes));
	}
}
