/*
 * $Id: KeyStoreTest.java,v 1.1 1999/01/25 00:44:52 wslade Exp $
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/KeyStoreTest.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/25 00:44:52 $
 * $State: Exp $
 */

package aba.test;

import java.io.*;
import java.util.*;
import java.security.*;

import au.net.aba.security.cert.Certificate;
import au.net.aba.security.*;
import au.net.aba.security.cert.*;
import au.net.aba.security.spec.*;

import au.net.aba.crypto.spec.AsciiEncodedKeySpec;

import javax.crypto.*;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class KeyStoreTest
{
	public static void main(String[] argv)
	{
		try
		{
			System.out.println("Starting Keystore test");
			
			if (argv.length != 1)
			{
				System.out.println("Usage:");
				System.out.println("java KeyStoreTest <certificatefile>");
				return;
			}
			
			Provider provider = new au.net.aba.crypto.provider.ABAProvider();
			java.security.Security.addProvider(provider);

			au.net.aba.security.KeyStore ks = au.net.aba.security.KeyStore.getInstance("ABA");
			test(ks);

			KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
			kpg.initialize(1024, new SecureRandom());
			KeyPair pair = kpg.generateKeyPair();
			PrivateKey pkey = pair.getPrivate();
			
			CertificateFactory cf = CertificateFactory.getInstance("X.509");

			InputStream is = new FileInputStream(argv[0]);
			
			Certificate cert = cf.generateCertificate(is);
			ks.setCertificateEntry("myfirst", cert);
			
			final Certificate[] certs = { cert };
			ks.setKeyEntry("mysecond", pkey, "0987654321".toCharArray(), certs);
			
			// store
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			
			System.out.println("beforeStore");
			ks.store(out, "1234567890".toCharArray());
			System.out.println("afterStore");
			
			byte[] buf = out.toByteArray();
			aba.test.ByteUtil.printHexBlock("encrypted", buf);
			
			// load
			InputStream in = new ByteArrayInputStream(out.toByteArray());
			System.out.println("beforeLoad");
			ks.load(in, "1234567890".toCharArray());
			System.out.println("afterLoad");
			
			test(ks);
			
			cert = ks.getCertificate("myfirst");
			
			System.out.println("Finished Keystore test");
		}
		catch(ExceptionInInitializerError e)
		{
			e.getException().printStackTrace();
		}
		catch(Throwable e)
		{
			e.printStackTrace();
		}
	}
	
	private static void test(au.net.aba.security.KeyStore store)
		throws Throwable
	{
		System.out.println("size = " + store.size());
		System.out.println("aliases = ");
		for( Enumeration enum = store.aliases();
			 enum.hasMoreElements();)
		{
			System.out.print(enum.nextElement().toString() + " ");			
		}
		System.out.println();
	}
}
