/*
 * $Id: PBECipherTest.java,v 1.1 1999/02/18 00:22:00 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/PBECipherTest.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/02/18 00:22:00 $
 * $State: Exp $
 */

package aba.test;

import java.io.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.security.*;
import au.net.aba.crypto.provider.ABAProvider;

/**
 * This test class is designed to exercise PBE (password based
 * encryption) ciphers.
 */
public class PBECipherTest
{
	public static void main(String[] args)
	throws Exception
	{
		Security.addProvider(new ABAProvider());
		SecureRandom random = new SecureRandom();

		byte[] salt = new byte[10];
		random.nextBytes(salt);

		PBEParameterSpec paramSpec = new PBEParameterSpec(salt, 10);
		char passwd[] = new char[args[3].length()];
		args[3].getChars(0, passwd.length, passwd, 0);
		PBEKeySpec keySpec = new PBEKeySpec(passwd);

		SecretKeyFactory keyFact = SecretKeyFactory.getInstance(
			args[0], args[1]);
		SecretKey key = keyFact.generateSecret(keySpec);

		Cipher encCipher = Cipher.getInstance(args[0], args[1]);
		Cipher decCipher = Cipher.getInstance(args[0], args[2]);

		byte[] plainText = args[4].getBytes();

		System.out.println("src: " + ByteUtil.bytesToHexStr(plainText));

		encCipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
		byte[] cipherText = encCipher.doFinal(plainText);

		System.out.println("enc: " + ByteUtil.bytesToHexStr(cipherText));

		decCipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
		byte[] decText = decCipher.doFinal(cipherText);

		System.out.println("dec: " + ByteUtil.bytesToHexStr(decText));

		boolean okay = true;
		for (int i = 0; i < plainText.length; i++)
		{
			if (plainText[i] != decText[i])
			{
				okay = false;
				break;
			}
		}

		if (okay)
		{
			System.out.println("Encryption okay");
		}
		else
		{
			System.out.println("Encryption failed");
		}
	}
}
