/*
 * $Id: RC4Test.java,v 1.8 1998/10/27 05:11:09 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/RC4Test.java,v $
 * $Revision: 1.8 $
 * $Date: 1998/10/27 05:11:09 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import java.security.*;
import javax.crypto.*;
import java.security.*;
import au.net.aba.crypto.provider.ABAProvider;

/**
 * Tester for the RC4 based cipher object.
 */
public class RC4Test
{
	public static void main(
		String[]	args)
		throws Exception
	{
		Key			key;
		KeyGenerator		keyGen;
		SecureRandom		rand;
		Cipher			in, out;
		CipherInputStream	cIn;
		CipherOutputStream	cOut;
		ByteArrayInputStream	bIn;
		ByteArrayOutputStream	bOut;
		int			nBits;
		ABAProvider		provider;

                provider = new ABAProvider();

                Security.addProvider(provider);
		
		rand = new SecureRandom();

		nBits = Integer.parseInt(args[0]);

		try
		{
			keyGen = KeyGenerator.getInstance("RC4", "ABA");

			keyGen.init(nBits, rand);

			key = keyGen.generateKey();

			in = Cipher.getInstance("RC4", "ABA");
			out = Cipher.getInstance("RC4", "ABA");
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			System.exit(2);
			return;
		}

		in.init(Cipher.DECRYPT_MODE, key);
		out.init(Cipher.ENCRYPT_MODE, key);

		/*
		 * encipher
		 */
		bOut = new ByteArrayOutputStream();

		cOut = new CipherOutputStream(bOut, out);

		System.out.println("orig: "
				+ ByteUtil.bytesToHexStr(args[1].getBytes()));

		try
		{
			cOut.write(args[1].getBytes());
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(3);
			return;
		}

		/*
		 * decipher
		 */
		byte[]	bytes;

		bytes = bOut.toByteArray();

		System.out.println("enc:  " + ByteUtil.bytesToHexStr(bytes));

		bIn = new ByteArrayInputStream(bytes);

		cIn = new CipherInputStream(bIn, in);

		try
		{
			cIn.read(bytes);
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(4);
			return;
		}

		System.out.println("dec:  " + ByteUtil.bytesToHexStr(bytes));
	}
}
