/*
 * $Id: RC4Test.java,v 1.5 1999/02/19 03:51:34 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/test/RC4Test.java,v $
 * $Revision: 1.5 $
 * $Date: 1999/02/19 03:51:34 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import javax.crypto.*;
import au.net.aba.security.*;

/**
 * Tester for the RC4 based cipher object.
 */
public class RC4Test
{
	public static void main(
		String[]	args)
		throws Exception
	{
		Key			key;
		KeyGenerator		keyGen;
		SecureRandom		rand;
		Cipher			in, out;
		CipherInputStream	cIn;
		CipherOutputStream	cOut;
		ByteArrayInputStream	bIn;
		ByteArrayOutputStream	bOut;
		int			nBits;
		
		rand = new SecureRandom();

		nBits = Integer.parseInt(args[0]);

		try
		{
			keyGen = KeyGenerator.getInstance("RC4", "ABA");

			keyGen.init(rand);

			key = keyGen.generateKey();

			in = Cipher.getInstance("RC4", "ABA");
			out = Cipher.getInstance("RC4", "ABA");
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			System.exit(2);
			return;
		}

		in.init(Cipher.DECRYPT_MODE, key);
		out.init(Cipher.ENCRYPT_MODE, key);

		/*
		 * encipher
		 */
		byte[]	bytes;

		bOut = new ByteArrayOutputStream();

		cOut = new CipherOutputStream(bOut, out);

		bytes = new byte[args[1].length()];
		args[1].getBytes(0, args[1].length(), bytes, 0);

		System.out.println("orig: "
				+ ByteUtil.bytesToHexStr(bytes));

		try
		{
			cOut.write(bytes);
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(3);
			return;
		}

		/*
		 * decipher
		 */

		bytes = bOut.toByteArray();

		System.out.println("enc:  " + ByteUtil.bytesToHexStr(bytes));

		bIn = new ByteArrayInputStream(bytes);

		cIn = new CipherInputStream(bIn, in);

		try
		{
			cIn.read(bytes);
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(4);
			return;
		}

		System.out.println("dec:  " + ByteUtil.bytesToHexStr(bytes));
	}
}
