/*
 * $Id: RSATest.java,v 1.13 1998/10/20 07:49:47 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/RSATest.java,v $
 * $Revision: 1.13 $
 * $Date: 1998/10/20 07:49:47 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import java.security.*;
import javax.crypto.*;
import au.net.aba.crypto.provider.ABAProvider;
import au.net.aba.crypto.spec.AsciiEncodedKeySpec;
import au.net.aba.security.KeyFactory;
import au.net.aba.security.spec.*;

/**
 * Tester for the RSA based cipher object.
 */
public class RSATest
{
	public static void main(
		String[]	args)
		throws KeyException
	{
		PublicKey		keyOut;
		PrivateKey		keyIn;
		Cipher			in, out;
		CipherInputStream	cIn;
		CipherOutputStream	cOut;
		ByteArrayInputStream	bIn;
		ByteArrayOutputStream	bOut;
		SecureRandom		rand;

		Security.addProvider(new ABAProvider());

		rand = new SecureRandom();
		
		try
		{
			KeyFactory fact = KeyFactory.getInstance("RSA", "ABA");

			AsciiEncodedKeySpec pubKeySpec = new AsciiEncodedKeySpec(args[0]);
			keyOut = fact.generatePublic(pubKeySpec);

			AsciiEncodedKeySpec privKeySpec = new AsciiEncodedKeySpec(args[1]);
			keyIn = fact.generatePrivate(privKeySpec);
		}
		catch (Exception e)
		{
			System.err.println(e);
			e.printStackTrace(System.err);
			System.exit(1);
			return;
		}

		try
		{
			in = Cipher.getInstance("RSA/ECB/PKCS1Padding", "ABA");
			out = Cipher.getInstance("RSA/ECB/PKCS1Padding", "ABA");
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
			System.exit(2);
			return;
		}

		in.init(Cipher.DECRYPT_MODE, keyIn);
		out.init(Cipher.ENCRYPT_MODE, keyOut, rand);

		/*
		 * encipher
		 */
		byte[]	bytes;

		bOut = new ByteArrayOutputStream();

		cOut = new CipherOutputStream(bOut, out);

		bytes = new byte[args[2].length()];

		for (int i = 0; i != bytes.length; i++)
		{
			bytes[i] = (byte)args[2].charAt(i);
		}

		System.out.println("orig: "
				+ ByteUtil.bytesToHexStr(bytes));

		/*
		 * cipher
		 */
		try
		{
			cOut.write(bytes);
			cOut.close();
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(3);
			return;
		}

		bytes = bOut.toByteArray();

		System.out.println("enc:  " + ByteUtil.bytesToHexStr(bytes));

		bIn = new ByteArrayInputStream(bytes);

		cIn = new CipherInputStream(bIn, in);

		byte[] finalBytes = null;

		try
		{
			DataInputStream dIn = new DataInputStream(cIn);
			finalBytes = new byte[args[2].length()];
			dIn.readFully(finalBytes);
		}
		catch (IOException e)
		{
			System.out.println("dec:  "
				+ ByteUtil.bytesToHexStr(finalBytes));
			e.printStackTrace();
			System.err.println(e.getMessage());
			System.exit(4);
			return;
		}

		System.out.println("dec:  "
				+ ByteUtil.bytesToHexStr(finalBytes));
	}
}
