/*
 * $Id: RSATest.java,v 1.8 1999/02/19 03:51:35 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/test/RSATest.java,v $
 * $Revision: 1.8 $
 * $Date: 1999/02/19 03:51:35 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import java.util.*;
import javax.crypto.*;
import au.net.aba.security.*;
import au.net.aba.security.spec.*;
import au.net.aba.crypto.spec.AsciiEncodedKeySpec;
import au.net.aba.math.BigInteger;

/**
 * Tester for the RSA based cipher object.
 */
public class RSATest
{
	public static void main(
		String[]	args)
		throws KeyException
	{
		PublicKey		keyOut;
		PrivateKey		keyIn;
		Cipher			in, out;
		CipherInputStream	cIn;
		CipherOutputStream	cOut;
		ByteArrayInputStream	bIn;
		ByteArrayOutputStream	bOut;

		try
		{
			KeyFactory fact = KeyFactory.getInstance("RSA", "ABA");

			AsciiEncodedKeySpec pubKeySpec = new AsciiEncodedKeySpec(args[0]);
			keyOut = fact.generatePublic(pubKeySpec);

			AsciiEncodedKeySpec privKeySpec = new AsciiEncodedKeySpec(args[1]);
			keyIn = fact.generatePrivate(privKeySpec);
		}
		catch (Exception e)
		{
			System.err.println(e);
			e.printStackTrace(System.err);
			System.exit(1);
			return;
		}

		try
		{
			in = Cipher.getInstance("RSA", "ABA");
			out = Cipher.getInstance("RSA", "ABA");
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
			System.exit(2);
			return;
		}

		in.init(Cipher.DECRYPT_MODE, keyOut);
		out.init(Cipher.ENCRYPT_MODE, keyIn, new SecureRandom());

		/*
		 * encipher
		 */
		byte[]	bytes;

		bOut = new ByteArrayOutputStream();

		cOut = new CipherOutputStream(bOut, out);

		bytes = new byte[args[2].length()];

		for (int i = 0; i != bytes.length; i++)
		{
			bytes[i] = (byte)args[2].charAt(i);
		}

		System.out.println("orig: "
				+ ByteUtil.bytesToHexStr(bytes));

		try
		{
			cOut.write(bytes);
			cOut.close();
		}
		catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(3);
			return;
		}

		/*
		 * cipher
		 */
		byte[]	finalBytes;
		int	count;

		bytes = bOut.toByteArray();

		System.out.println("enc:  " + ByteUtil.bytesToHexStr(bytes));

		bIn = new ByteArrayInputStream(bytes);

		cIn = new CipherInputStream(bIn, in);

		try
		{
			DataInputStream din = new DataInputStream(cIn);
			finalBytes = new byte[args[2].length()];
			din.readFully(finalBytes);
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.err.println(e.getMessage());
			System.exit(4);
			return;
		}

		System.out.println("dec:  "
				+ ByteUtil.bytesToHexStr(finalBytes));
	}
}
