/*
 * $Id: SHA1Test.java,v 1.2 1998/10/20 07:49:48 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/test/SHA1Test.java,v $
 * $Revision: 1.2 $
 * $Date: 1998/10/20 07:49:48 $
 * $State: Exp $
 */
package aba.test;

import java.io.*;
import java.security.*;
import javax.crypto.*;
import au.net.aba.crypto.provider.ABAProvider;

/**
 * Test class for SHA1
 */
public class SHA1Test
{
	final static String[] plain =
		{ "abc", "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq" };

	final static byte[][] digests = {
		{ (byte)0xa9, (byte)0x99, (byte)0x3e, (byte)0x36, (byte)0x47, (byte)0x06, (byte)0x81, (byte)0x6a,
		(byte)0xba, (byte)0x3e, (byte)0x25, (byte)0x71, (byte)0x78, (byte)0x50, (byte)0xc2, (byte)0x6c, (byte)0x9c, (byte)0xd0, (byte)0xd8, (byte)0x9d},
		{ (byte)0x84, (byte)0x98, (byte)0x3e, (byte)0x44, (byte)0x1c, (byte)0x3b, (byte)0xd2, (byte)0x6e,
		(byte)0xba, (byte)0xae, (byte)0x4a, (byte)0xa1, (byte)0xf9, (byte)0x51, (byte)0x29, (byte)0xe5, (byte)0xe5, (byte)0x46, (byte)0x70, (byte)0xf1}
	};

	final static byte[] big_digest = {
		(byte)0x34, (byte)0xaa, (byte)0x97, (byte)0x3c, (byte)0xd4, (byte)0xc4, (byte)0xda, (byte)0xa4,
		(byte)0xf6, (byte)0x1e, (byte)0xeb, (byte)0x2b, (byte)0xdb, (byte)0xad, (byte)0x27, (byte)0x31, (byte)0x65, (byte)0x34, (byte)0x01, (byte)0x6f
	};

	public static void main(
		String[]	args)
		throws KeyException
	{
		MessageDigest		digest;

		Security.addProvider(new ABAProvider());

		try
		{
			digest = MessageDigest.getInstance("SHA-1", "ABA");
		}
		catch (Exception e)
		{
			System.err.println(e);
			e.printStackTrace(System.err);
			System.exit(1);
			return;
		}

		for (int i = 0; i < plain.length; i++)
		{
			digest.update(plain[i].getBytes());
			byte[] res = digest.digest();

			String resString = ByteUtil.bytesToHexStr(res).toLowerCase();
			String digString = ByteUtil.bytesToHexStr(digests[i]).toLowerCase();

			System.out.println(plain[i].toLowerCase()
				+ ":" + resString
				+ ":" + digString
				+ ":" + resString.equals(digString));
		}

		for (int i = 0; i < 1000*1000; i++)
		{
			digest.update((byte)'a');
		}

		byte[] res = digest.digest();
		String resString = ByteUtil.bytesToHexStr(res).toLowerCase();
		String digString = ByteUtil.bytesToHexStr(big_digest).toLowerCase();

		System.out.println("1000x(1000xa)"
			+ ":" + resString
			+ ":" + digString
			+ ":" + resString.equals(digString));
	}
}
