/*
 * $Id: KeyGeneratorSpi.java,v 1.5 1998/10/26 01:47:00 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/KeyGeneratorSpi.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/26 01:47:00 $
 * $State: Exp $
 */
package javax.crypto;

import java.security.*;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * This class defines the Service Provider Interface (SPI) for the KeyGenerator
 * class. All the abstract methods in this class must be implemented by each
 * cryptographic service provider who wishes to supply the implementation of a
 * key generator for a particular algorithm. 
 * 
 * @see SecretKey
 */
public abstract class KeyGeneratorSpi
{
	public final static String ident = "$Id: KeyGeneratorSpi.java,v 1.5 1998/10/26 01:47:00 leachbj Exp $";

	/**
	 * Creates a KeyGeneratorSpi object.
	 */
	public KeyGeneratorSpi()
	{
	}

	/**
	 * Initialises the key generator. 
	 *
	 * @param random the source of randomness for this generator.
	 */
	protected abstract void engineInit(SecureRandom random);
	
	/**
	 * Initialises the key generator with the specified parameter set
	 * and a user-provided source of randomness. 
	 *
	 * @param params the key generation parameters
	 * @param random the source of randomness for this key generator
	 * @exception InvalidAlgorithmParameterException if params is
	 *	inappropriate for this key generator.
	 */
	protected abstract void engineInit(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidAlgorithmParameterException;

	/**
	 * Initialises this key generator for a certain strength, using the
	 * given source of randomness.
	 *
	 * @param strength the strength of the key. This is an algorithm-specific
	 *     metric specified in number of bits.
	 * @param random the source of randomness for this key generator
	 */
	protected abstract void engineInit(
		int		strength,
		SecureRandom	random);

	/**
	 * Generates a secret key. 
	 *
	 * @return the new key 
	 */
	protected abstract SecretKey engineGenerateKey();
}
