/*
 * $Id: NullCipher.java,v 1.8 1998/10/26 01:47:02 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/NullCipher.java,v $
 * $Revision: 1.8 $
 * $Date: 1998/10/26 01:47:02 $
 * $State: Exp $
 */
package javax.crypto;

import java.security.*;
import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * The NullCipher class is a class that provides an
 * "identity cipher" -- one that does not transform the
 * plaintext. As a consequence, the ciphertext is identical
 * to the plaintext. All initialisation methods do nothing,
 * while the blocksize is set to 1 byte. 
 */
public class NullCipher extends Cipher
{
	public final static String ident = "$Id: NullCipher.java,v 1.8 1998/10/26 01:47:02 leachbj Exp $";

	private static NullCipherSpi spi = new NullCipherSpi();
	private static Provider provider = null;
	private static String transformation = "";

	/**
	 *
	 */
	public NullCipher()
	{
		super(spi, provider, transformation);
	}

	/**
	 * The NullCipher class is a class that provides an
	 * "identity cipher" -- one that does not tranform the
	 * plaintext. As a consequence, the ciphertext is identical
	 * to the plaintext. All initialisation methods do nothing,
	 * while the blocksize is set to 1 byte. 
	 */
	static private class NullCipherSpi extends CipherSpi
	{
		public final static String ident = "$Id: NullCipher.java,v 1.8 1998/10/26 01:47:02 leachbj Exp $";

		public NullCipherSpi()
		{
		}

		/**
		 * Sets the mode of this cipher.  Ignored.
		 *
		 * @param mode the cipher mode 
		 * @exception NoSuchAlgorithmException	if the requested cipher mode
		 *					does not exist 
		 */
		public void engineSetMode(String mode)
			throws NoSuchAlgorithmException
		{
		}

		/**
		 * Sets the padding mechanism of this cipher. Ignored.
		 *
		 * @param padding the padding mechanism
		 * @exception NoSuchPaddingException	if the requested padding 
		 *					mechanism does not exist.
		 */
		public void engineSetPadding(String padding)
			throws NoSuchPaddingException
		{
		}

		/**
		 * Returns the block size (in bytes).   In this case 1.
		 *
		 * @return always returns 1.
		 */
		protected int engineGetBlockSize()
		{
			return 1;
		}

		/**
		 * Returns the length in bytes that an output buffer would need to be
		 * in order to hold the result of the next update or doFinal operation,
		 * given the input length inputLen (in bytes). 
		 * <p>
		 * This method will always return the value of inputLen.
		 *
		 * @param inputLen the input length (in bytes) 
		 * @return the required output buffer size (in bytes)
		 */
		protected int engineGetOutputSize(int inputLen)
		{
			return inputLen;
		}

		/**
		 * Returns the initialisation vector (IV) in a new buffer. 
		 *
		 * @return null - this cipher does not us an IV.
		 */
		protected byte[] engineGetIV()
		{
			return null;
		}

		/**
		 * This method does nothing.
		 * 
		 * @param opmode the operation mode of this cipher (this is either
		 *			ENCRYPT_MODE or DECRYPT_MODE) 
		 * @param key the encryption key 
		 * @param random the source of randomness 
		 *
		 * @exception InvalidKeyException if the given key is inappropriate
		 *			for initialising this cipher 
		 */
		protected void engineInit(int opmode,
							Key key, SecureRandom random)
			throws InvalidKeyException
		{
		}

		/**
		 * This method does nothing.
		 * 
		 * @param opmode the operation mode of this cipher (this is either
		 *			ENCRYPT_MODE or DECRYPT_MODE) 
		 * @param key the encryption key 
		 * @param params the algorithm parameters
		 * @param random the source of randomness 
		 *
		 * @exception InvalidKeyException if the given key is inappropriate
		 *			for initialising this cipher 
		 * @exception InvalidAlgorithmParameterException if the given algorithm
		 *     parameters are inappropriate for this cipher 
		 */
		protected void engineInit(int opmode,
				Key key, AlgorithmParameterSpec params,
							SecureRandom random)
			throws InvalidKeyException, InvalidAlgorithmParameterException
		{
		}

		/**
		 * This method does nothing.
		 * <p>
		 * Initialises this cipher with a key, a set of algorithm
		 * parameters, and a source of randomness. 
		 * <p>
		 * The cipher is initialised for encryption or decryption,
		 * depending on the value of opmode. 
		 * <p>
		 * If this cipher (including its underlying feedback or padding
		 * scheme) requires any random bytes, it will get them from
		 * random. 
		 * <p>
		 * Note that when a Cipher object is initialised, it loses all
		 * previously-acquired state. In other words, initialising a
		 * Cipher is equivalent to creating a new instance of that Cipher
		 * and initialising it.
		 * 
		 * @param opmode the operation mode of this cipher (this is either
		 *    ENCRYPT_MODE or DECRYPT_MODE)
		 * @param key the encryption key
		 * @param params the algorithm parameters
		 * @param random the source of randomness
		 * @exception InvalidKeyException if the given key is
		 *    inappropriate for initialising this cipher
		 * @exception InvalidAlgorithmParameterException if the given
		 *    algorithm parameters are inappropriate for this cipher
		 */
		protected void engineInit(int opmode, Key key,
			AlgorithmParameters params, SecureRandom random)
			throws InvalidKeyException, InvalidAlgorithmParameterException
		{
		}

		/**
		 * just a stub. Normally returns the algorithm parameters, but
		 * there aren't any!
		 */
		protected AlgorithmParameters engineGetParameters()
		{
			return null;
		}

		/**
		 * Copies the input buffer to the output buffer.
		 *
		 * @param input the input buffer 
		 * @param inputOffset the offset in input where the input starts 
		 * @param inputLen the input length 
		 * @return the new buffer with the result, or null if the underlying
		 *    cipher is a block cipher and the input data is too short to
		 *    result in a new block. 
		 */
		protected byte[] engineUpdate(
						byte[] in, int inOff, int inLen)
		{
			byte[] result = new byte[inLen];
			System.arraycopy(in, inOff, result, 0, inLen);

			return result;
		}

		/**
		 * Copies the requested number of bytes from the input
		 * buffer to the output buffer.
		 * 
		 * @param input the input buffer 
		 * @param inputOffset the offset in input where the input starts 
		 * @param inputLen the input length 
		 * @param output the buffer for the result 
		 * @param outputOffset the offset in output where the result is stored 
		 * 
		 * @exception  ShortBufferException if the given output buffer is too
		 *     small to hold the result 
		 *
		 * @return the number of bytes stored in output 
		 */
		protected int engineUpdate(
						byte[] in, int inOff, int inLen,
								byte[] out, int outOff)
			throws ShortBufferException
		{
			if (out.length - outOff < inLen)
			{
				throw new ShortBufferException("output buffer too short.");
			}

			System.arraycopy(in, inOff, out, outOff, inLen);

			return inLen;
		}

		/**
		 * See engineUpdate().
		 *
		 * @param input the input buffer 
		 * @param inputOffset the offset in input where the input starts 
		 * @param inputLen the input length 
		 *
		 * @exception IllegalBlockSizeException if this cipher is a block
		 * 	cipher, no padding has been requested (only in encryption
		 *	mode), and the total input length of the data processed by
		 *	this cipher is not a multiple of block size 
		 * @exception BadPaddingException if this cipher is in decryption mode,
		 * 	and (un)padding has been requested, but the decrypted data is
		 *	not bounded by the appropriate padding bytes.
		 *
		 * @return the new buffer with the result 
		 */
		protected byte[] engineDoFinal(
					byte input[], int inputOffset, int inputLen)
			throws IllegalBlockSizeException, BadPaddingException
		{
			// there is no buffering in this cipher
			if (input != null)
			{
				return engineUpdate(input, inputOffset, inputLen);
			}

			return null;
		}
	
		/**
		 *  See above.
		 * 
		 * @param input the input buffer 
		 * @param inputOffset - the offset in input where the input starts 
		 * @param inputLen - the input length 
		 * @param output - the buffer for the result 
		 * @param outputOffset - the offset in output where the result is stored
		 * 
		 * @exception IllegalBlockSizeException if this cipher is a block
		 *    cipher, no padding has been requested (only in encryption mode),
		 *    and the total input length of the data processed by this cipher
		 *    is not a multiple of block size 
		 * @exception ShortBufferException if the given output buffer is too
		 *    small to hold the result 
		 * @exception BadPaddingException if this cipher is in decryption mode,
		 *     and (un)padding has been requested, but the decrypted data is
		 *     not bounded by the appropriate padding bytes 
		 * @returns the number of bytes stored in output 
		 */
		protected int engineDoFinal(
					byte[] in, int inOff, int inLen,
								byte[] out, int outOff)
			throws ShortBufferException, IllegalBlockSizeException,
								BadPaddingException
		{
			// there is no buffering in this cipher
			if (in != null)
			{
				return engineUpdate(in, inOff, inLen, out, outOff);
			}

			return 0;
		}
	}
}
