/*
 * $Id: DHGenParameterSpec.java,v 1.4 1998/10/28 23:20:02 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/DHGenParameterSpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/28 23:20:02 $
 * $State: Exp $
 */

package javax.crypto.spec;

import au.net.aba.security.spec.*;

/**
 * This class specifies the set of parameters used for generating Diffie-Hellman
 * (system) parameters for use in Diffie-Hellman key agreement. This is
 * typically done by a central authority. 
 * <p>
 * The central authority, after computing the parameters, must send this
 * information to the parties looking to agree on a secret key. 
 *
 * @see java.security.spec.AlgorithmParameterSpec
 * @see java.security.AlgorithmParameterGenerator
 * @see DHParameterSpec
 */
public class DHGenParameterSpec
	implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: DHGenParameterSpec.java,v 1.4 1998/10/28 23:20:02 leachbj Exp $";

	private int	primeSize;
	private int	exponentSize;

	/**
	 * basic constructor
	 */
	public DHGenParameterSpec(
		int primeSize,
		int exponentSize)
	{
		this.primeSize = primeSize;
		this.exponentSize = exponentSize;
	}

	/**
	 * Returns the size in bits of the prime modulus.
	 *
	 * @return the size in bits of the prime modulus
	 */
	public int getPrimeSize()
	{
		return primeSize;
	}

	/**
	 * Returns the size in bits of the random exponent (private value).
	 *
	 * @return the size in bits of the random exponent (private value)
	 */
	public int getExponentSize()
	{
		return exponentSize;
	}
}
