/*
 * $Id: DHPrivateKeySpec.java,v 1.4 1998/10/28 23:20:03 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/DHPrivateKeySpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/28 23:20:03 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.math.*;
import au.net.aba.security.spec.*;

/**
 * This class specifies a Diffie-Hellman private key with its associated
 * parameters. 
 *
 * @see java.security.Key
 * @see java.security.KeyFactory
 * @see java.security.spec.KeySpec
 * @see java.security.spec.PKCS8EncodedKeySpec
 * @see DHPublicKeySpec
 */
public class DHPrivateKeySpec
	implements KeySpec
{
	public final static String ident = "$Id: DHPrivateKeySpec.java,v 1.4 1998/10/28 23:20:03 leachbj Exp $";

	private BigInteger	x;	// the private value
	private BigInteger	p;	// the prime modulus
	private BigInteger	g;	// the base generator

	/**
	 * standard constructor.
	 *
	 * @param x the private value.
	 * @param p the prime modulus.
	 * @param g the base generator.
	 */
	public DHPrivateKeySpec(
		BigInteger	x,
		BigInteger	p,
		BigInteger	g)
	{
		this.x = x;
		this.p = p;
		this.g = g;
	}

	/**
	 * Returns the private value x.
	 *
	 * @return the private value x
	 */
	public BigInteger getX()
	{
		return x;
	}

	/**
	 * Returns the prime modulus p.
	 *
	 * @return the prime modulus p
	 */
	public BigInteger getP()
	{
		return p;
	}

	/**
	 * Returns the base generator g.
	 *
	 * @return the base generator g
	 */
	public BigInteger getG()
	{
		return g;
	}
}
