/*
 * $Id: DHPublicKeySpec.java,v 1.4 1998/10/28 23:20:03 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/DHPublicKeySpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/28 23:20:03 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.math.*;

import java.security.spec.*;

/**
 * This class specifies a Diffie-Hellman public key with its associated
 * parameters. 
 *
 * @see DHPrivateKeySpec
 */
public class DHPublicKeySpec
	implements KeySpec
{
	public final static String ident = "$Id: DHPublicKeySpec.java,v 1.4 1998/10/28 23:20:03 leachbj Exp $";

	private BigInteger	y;	// the public value
	private BigInteger	p;	// the prime modulus
	private BigInteger	g;	// the base generator

	/**
	 * basic constructor.
	 *
	 * @param y the public value.
	 * @param p the prime modulus.
	 * @param g the base generator.
	 */
	public DHPublicKeySpec(
		BigInteger	y,
		BigInteger	p,
		BigInteger	g)
	{
		this.y = y;
		this.p = p;
		this.g = g;
	}

	/**
	 * Returns the public value y.
	 *
	 * @return the public value y
	 */
	public BigInteger getY()
	{
		return y;
	}

	/**
	 * Returns the prime modulus p.
	 *
	 * @return the prime modulus p
	 */
	public BigInteger getP()
	{
		return p;
	}

	/**
	 * Returns the base generator g.
	 *
	 * @return the base generator g
	 */
	public BigInteger getG()
	{
		return g;
	}
}
