/*
 * $Id: IvParameterSpec.java,v 1.5 1998/10/26 01:47:06 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/IvParameterSpec.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/26 01:47:06 $
 * $State: Exp $
 */

package javax.crypto.spec;

import au.net.aba.security.spec.*;

/**
 * This class specifies an initialisation vector (IV). IVs are used by
 * ciphers in feedback mode, e.g., DES in CBC mode. 
 *
 * @see AlgorithmParameterSpec
 */
public class IvParameterSpec implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: IvParameterSpec.java,v 1.5 1998/10/26 01:47:06 leachbj Exp $";

	private byte[]	iv;

	/**
	 * Uses the bytes in iv as the IV.
	 *
         * @param iv the buffer with the IV 
	 */
	public IvParameterSpec(
		byte[] iv)
	{
		this.iv = new byte[iv.length];
		System.arraycopy(iv, 0, this.iv, 0, iv.length);
	}

	/**
	 * Uses the first len bytes in iv, beginning at offset, as the IV.
	 *
	 * @param iv the buffer with the IV 
	 * @param offset the offset in iv, where the IV starts 
	 * @param len the number of IV bytes 
	 */
	public IvParameterSpec(
		byte[] iv,
		int offset,
		int len)
	{
		this.iv = new byte[len];
		System.arraycopy(iv, offset, this.iv, 0, len);
	}

	/**
	 * Returns the initialisation vector (IV).
	 *
	 * @return the initialisation vector (IV)
	 */
	public byte[] getIV()
	{
		byte[]	tmp;

		tmp = new byte[iv.length];
		System.arraycopy(iv, 0, tmp, 0, iv.length);

		return tmp;
	}
}
