/*
 * $Id: PBEParameterSpec.java,v 1.3 1998/10/05 04:50:19 dgh Exp $
 * $Author: dgh $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/PBEParameterSpec.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/05 04:50:19 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.security.spec.*;

/**
 * This class specifies the set of parameters used with password-based
 * encryption (PBE), as defined in the PKCS #5 standard. 
 *
 * @see AlgorithmParameterSpec
 */
public class PBEParameterSpec
	implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: PBEParameterSpec.java,v 1.3 1998/10/05 04:50:19 dgh Exp $";

	private byte[]	salt;
	private int	iterationCount;

	/**
	 * basic constructor
	 *
	 * @param salt the byte array to use as a salt.
	 * @param iterationCount the iterationCount.
	 */
	public PBEParameterSpec(
		byte[] salt,
		int iterationCount)
	{
		this.salt = salt;
		this.iterationCount = iterationCount;
	}

	/**
	 * Returns the salt.
	 *
	 * @return the salt.
	 */
	public byte[] getSalt()
	{
		return salt;
	}

	/**
	 * Returns the iteration count.
	 *
	 * @return the iteration count
	 */
	public int getIterationCount()
	{
		return iterationCount;
	}
}
